# -*- python -*-
# -*- coding: utf-8 -*-
#
#  This file is part of dreamtools software
#
#  Copyright (c) 2013-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.com/dreamtools
#
##############################################################################

import json
import os
import pandas as pd
from dreamtools.dream8.D8C1 import submissions
from dreamtools.dream8.D8C1 import scoring
from dreamtools.dream8.D8C1 import d8c1path

class Ranking(object):

    def __init__(self, name):
        self.name = name
        self.yours = 'YOUR_SUBMISSION'

    def _getdata(self, filename):
        path2data = os.path.split(os.path.abspath(__file__))[0]
        return os.sep.join([path2data, 'data', filename])

    def add_team(self):
        raise NotImplementedError

    def check_submissions(self):
        if getattr(self, 'submissions') is False:
            print("Call load_data_from_synapse() first. You 'll need synapse access as admin")

    def load_data_from_synapse(self):
        if self.name == 'SC1A':
            subs = submissions.SC1ASubmissions()
        elif self.name == 'SC1B':
            subs = submissions.SC1BSubmissions()
        elif self.name == 'SC2A':
            subs = submissions.SC2ASubmissions()
        elif self.name == 'SC2B':
            subs = submissions.SC2BSubmissions()
        subs.load_submissions()
        self.submissions = subs.submissions

    def get_ranked_df(self):
        df = self.ranked_df.copy()
        df = self._sort_df(df)
        return df

    def _sort_df(self, df):
        raise NotImplementedError

    def get_rank_your_submission(self):
        try:
            return list(self.get_ranking().index).index(self.yours) + 1
        except:
            print("Are we here?")
            try:
                return self.get_ranking().ix[self.yours].values[0]
            except:
                print("Use append_submission() method to add your submission")
                return 99

    def __str__(self):
        return self.get_ranked_df().to_string()


class SC1A_ranking(Ranking):
    """


    """
    def __init__(self):
        super(SC1A_ranking, self).__init__('SC1A')
        self.aucs = pd.read_json(self._getdata("SC1A_aucs.json"))
        self.ranked_df = pd.read_json(self._getdata("SC1A_results.json"))

    def _sort_df(self, df):
        df.sort('Mean Rank', inplace=True)
        df = df[['Final Rank', 'Team Name', 'Team Id', 'Submission Id',
                 'Entity Id', 'Mean AUC', 'Mean Rank']]
        df = df.reset_index(drop=True)
        df['Final Rank'] = [1+x for x in df.index]
        return df

    def get_aucs(self):
        return self.aucs

    def get_ranked_df_from_submission(self):
        """No need to be used. Just for admin to recreate SC1A_results.json"""
        self.check_submissions()

        submitterAlias = [sub['submitterAlias'] for sub in self.submissions]
        subId = [sub['substatus']['id'] for sub in self.submissions]
        entityId = [sub['substatus']['entityId'] for sub in self.submissions]
        userId = [sub['userId'] for sub in self.submissions]
        auc = [sub['mean_aucs'] for sub in self.submissions]
        ranking = [sub['ranking'] for sub in self.submissions]

        submitterAlias = [sub.replace("ChaosLab", "FreiburgBiossX") for sub in submitterAlias]

        df = pd.DataFrame({
            'Final Rank': ranking,
            'Team Name': submitterAlias,
            'Team Id': userId,
            'Submission Id': subId,
            'Entity Id': entityId,
            'Mean AUC': auc,
            'Mean Rank':ranking
        })
        df = self._sort_df(df)
        return df

    def get_aucs_from_submissions(self):
        """No need to be used. Just for admin to recreate SC1A_aucs.json"""
        self.check_submissions()
        df = self.get_ranked_df_from_submissions()
        data = [json.loads(this['substatus']['report']) for this in self.submissions]

        columns = [k1+"_"+k2 for k1 in data[0].keys() for k2 in data[0][k1].keys()]
        aucs = [[datum[k1][k2] for k1 in datum.keys() for k2 in datum[k1].keys()] for datum in data]

        #indices = df['Team Name']
        indices = [sub['submitterAlias'] for sub in self.submissions]
        indices = [this.replace("ChaosLab", "FreiburgBiossX") for this in indices]
        aucs = pd.DataFrame(aucs, columns=columns, index=indices)

        return aucs

    def get_ranking(self):
        aucs = self.get_aucs().copy()  # copy is essnetial since we then delete columns
        # some combo were removed in the final LB

        del aucs['BT549_NRG1']
        del aucs['BT20_Insulin']

        df = aucs.rank(ascending=False, method='min').mean(axis=1)
        df.sort()
        return df

    def __str__(self):
        return self.get_ranked_df().to_string()

    def append_submission(self, res):
        try:
            res = scoring.HPNScoringNetwork(res)
            res.compute_all_aucs()
        except Exception:
            if getattr(res, 'aucs') is False:
                raise Exception

        self.aucs.ix[self.yours] = [1] * 32
        #now replace the values as expected; s.auc is a 4 by  8 matrix
        for cell in res.auc.keys():
            for stim in res.auc[cell].keys():
                self.aucs.ix[self.yours][cell + "_" + stim] = res.auc[cell][stim]


class SC1B_ranking(Ranking):

    def __init__(self):
        super(SC1B_ranking, self).__init__('SC1B')
        self.ranked_df = pd.read_json(self._getdata("SC1B_results.json"))

    def _sort_df(self, df):
        df.sort('AUC', inplace=True, ascending=False)
        df = df.reset_index(drop=True)
        df['Final Rank'] = df.index
        df = df[['Final Rank', 'Team Name', 'Team Id', 'Submission Id',
                 'Entity Id', 'AUC', 'zscore']]
        return df

    def get_ranked_df_from_submissions(self):
        self.check_submissions()
        report = [sub['substatus']['report'] for sub in self.submissions]
        submitterAlias = [sub['submitterAlias'] for sub in self.submissions]
        subId = [sub['substatus']['id'] for sub in self.submissions]
        entityId = [sub['substatus']['entityId'] for sub in self.submissions]
        userId = [sub['userId'] for sub in self.submissions]

        auc = [sub['auc'] for sub in self.submissions]
        zscore = [json.loads(sub['substatus']['report'])['score'] for sub in self.submissions]

        submitterAlias = [sub.replace("ChaosLab", "FreiburgBiossX") for sub in submitterAlias]

        df = pd.DataFrame({
            'Team Name': submitterAlias,
            'Team Id': userId,
            'Submission Id': subId,
            'Entity Id': entityId,
            'AUC': auc,
            'zscore':zscore,
        })
        df = self._sort_df(df)
        return df

    def get_ranking(self):
        df = self.get_ranked_df()
        df = df[['Team Name', 'AUC']]
        df.set_index('Team Name', inplace=True)
        df = df.rank(ascending=False, method='min')
        return df

    def append_submission(self, res):
        try:
            res = scoring.HPNScoringNetworkInsilico(res)
            res.compute_score()
        except:
            pass

        ts = self.ranked_df.ix[0].copy()
        ts.name = 99
        ts['AUC'] = res.auc
        ts['Team Name'] = self.yours
        self.ranked_df = self._sort_df(self.ranked_df.append(ts))


class SC2A_ranking(Ranking):
    """

    """
    # 375805/alphabeta is a test from TC
    # 1991105/sakev from week 5 has different id from sakev week 6. renmove
    #      week5 that has a lower score anyway
    # 1971259/HD systems see SC1A function docstring
    userIds_toremove =  ["375805", "1991105", "1971259"]
    def __init__(self):
        super(SC2A_ranking, self).__init__('SC2A')
        self.rmses = pd.read_json(self._getdata("SC2A_rmses.json"))
        self.ranked_df = pd.read_json(self._getdata("SC2A_results.json"))

        self.phosphos_to_exclude = {
                'MCF7': ['TAZ_pS89', 'FOXO3a_pS318_S321', 'mTOR_pS2448'],
                'UACC812': ['TAZ_pS89','mTOR_pS2448'],
                'BT20': ['TAZ_pS89', 'FOXO3a_pS318_S321', 'mTOR_pS2448'],
                'BT549': ['TAZ_pS89','mTOR_pS2448']
        }

    def _sort_df(self, df):
        df.sort('Mean Rank', inplace=True)
        df = df[['Final Rank', 'Team Name', 'Team Id', 'Submission Id',
                 'Entity Id', 'Mean RMSE', 'Mean Rank']]
        df = df.reset_index(drop=True)
        df['Final Rank'] = [1+x for x in df.index]
        return df

    def get_ranked_df_from_submissions(self):
        self.check_submissions()
        submitterAlias = [sub['submitterAlias'] for sub in self.submissions]
        subId = [sub['substatus']['id'] for sub in self.submissions]
        entityId = [sub['substatus']['entityId'] for sub in self.submissions]
        userId = [sub['userId'] for sub in self.submissions]
        rmse = [sub['mean_rmse'] for sub in self.submissions]
        ranking = [sub['ranking'] for sub in self.submissions]

        df = pd.DataFrame({
            'Final Rank': ranking,
            'Team Name': submitterAlias,
            'Team Id': userId,
            'Submission Id': subId,
            'Entity Id': entityId,
            'Mean RMSE': rmse,
            'Mean Rank':ranking
        })
        df = self._sort_df(df)
        return df

    def get_rmses_from_submissions(self):
        df = self.get_ranked_df()
        data = [this['rmses'] for this in self.submissions]
        columns = [k1+"_"+k2 for k1 in data[0].keys() for k2 in data[0][k1].keys()]
        rmses = [[datum[k1][k2] for k1 in datum.keys() for k2 in datum[k1].keys()] for datum in data]
        indices = [sub['submitterAlias'] for sub in self.submissions]
        rmses = pd.DataFrame(rmses, columns=columns, index=indices)

        return rmses

    def get_rmses(self):
        return self.rmses.copy()

    def get_ranking(self):
        rmses = self.get_rmses()
        # Here we use ascending True because we have RMSE (lower=better)
        df = rmses.rank(ascending=True, method='min').mean(axis=1)
        df.sort()
        return df

    def append_submission(self, res):
        try:
            res = scoring.HPNScoringPrediction(res)
            res.compute_all_rmse()
        except:
            pass

        self.rmses.ix[self.yours] = [1] * 162
        #now replace the values as expected; s.auc is a 4 by  8 matrix
        for cell in res.rmse.keys():
            for stim in res.rmse[cell].keys():
                self.rmses.ix[self.yours][cell + "_" + stim] = res.rmse[cell][stim]



class SC2B_ranking(Ranking):
    """

    subs = submissions.SC1ASubmissions()
    subs.load_submissions()

    ranking = SC1A_ranking(subs.submissions)

    """


    # 375805/alphabeta is a test from TC
    # 1991105/sakev from week 5 has different id from sakev week 6. renmove
    #      week5 that has a lower score anyway
    # 1971259/HD systems see SC1A function docstring
    userIds_toremove =  ["375805", "1991105"]
    def __init__(self):
        super(SC2B_ranking, self).__init__('SC2B')
        self.rmses = pd.read_json(self._getdata("SC2B_rmses.json"))
        self.ranked_df = pd.read_json(self._getdata("SC2B_results.json"))

    def _sort_df(self, df):
        df.sort('Mean Rank', inplace=True)
        df = df[['Final Rank', 'Team Name', 'Team Id', 'Submission Id',
                 'Entity Id', 'Mean RMSE', 'Mean Rank']]

        df = df.reset_index(drop=True)
        df['Final Rank'] = [1+x for x in df.index]
        return df

    def get_ranked_df_from_submissions(self):
        self.check_submissions()
        submitterAlias = [sub['submitterAlias'] for sub in self.submissions]
        subId = [sub['substatus']['id'] for sub in self.submissions]
        entityId = [sub['substatus']['entityId'] for sub in self.submissions]
        userId = [sub['userId'] for sub in self.submissions]
        rmse = [sub['mean_rmse'] for sub in self.submissions]
        ranking = [sub['ranking'] for sub in self.submissions]

        df = pd.DataFrame({
            'Final Rank': ranking,
            'Team Name': submitterAlias,
            'Team Id': userId,
            'Submission Id': subId,
            'Entity Id': entityId,
            'Mean RMSE': rmse,
            'Mean Rank':ranking
        })
        df = self._sort_df(df)
        return df

    def get_rmses_from_submissions(self):
        df = self.get_ranked_df()
        data = [this['rmses'] for this in self.submissions]
        columns = [k1+"_"+k2 for k1 in data[0].keys() for k2 in data[0][k1].keys()]
        rmses = [[datum[k1][k2] for k1 in datum.keys() for k2 in datum[k1].keys()] for datum in data]
        indices = [sub['submitterAlias'] for sub in self.submissions]
        rmses = pd.DataFrame(rmses, columns=columns, index=indices)
        # recompute the mean RMSEs gives same results as in get_ranked_df
        # self.get_rmses().mean(axis=1)
        return rmses

    def get_rmses(self):
         return self.rmses.copy()

    def get_ranking(self):
        #I check on the old/wrong  GS network that we retrieve the rank and mean RMSE from the
        # LB so this code is correct. Using the new correct GS network, we of course
        # get different results but the results should be correct.
        rmses = self.get_rmses()
        # Here we use ascending True because we have RMSE (lower=better)
        df = rmses.rank(ascending=True, method='min').mean(axis=1)
        df.sort()
        return df

    def append_submission(self, res):
        try:
            res = scoring.HPNScoringPredictionInsilico(res)
            res.compute_all_rmse()
        except:
            pass

        self.rmses.ix[self.yours] = [1] * 231
        #now replace the values as expected; s.auc is a 4 by  8 matrix
        for cell in res.rmse.keys():
            for stim in res.rmse[cell].keys():
                self.rmses.ix[self.yours][cell + "_" + stim] = res.rmse[cell][stim]



class Aggregate():
    """
    a = ranking.SC1A_ranking()
    b = ranking.SC1B_ranking()
    aggr = Aggregate(a,b)
    print(aggr)

    """
    def __init__(self, rsca, rscb, name):
        """

        :param rsca:
        :param rscb:
        :param name: SC1 or SC2
        :return:
        """
        self.rsca = rsca
        self.rscb = rscb
        assert name in ['SC1', 'SC2']
        self.name = name

    def get_ranking(self):
        fillna = 1e6
        # Here we want to replace values that are not between 1 and N
        # into integer from 1 to N so that it can be compared to df1b
        dfa = self.rsca.get_ranking()
        N = len(dfa)
        dfa = pd.DataFrame(range(1, N+1), index=dfa.index)

        dfb = self.rscb.get_ranking()
        N2 = len(dfb)
        dfb = pd.DataFrame(range(1, N2+1), index=dfb.index)

        df = pd.concat([dfa, dfb], axis=1).fillna(fillna)
        df.columns = [self.name +'A', self.name +'B']
        df['mean'] = df.mean(axis=1)
        df.sort(columns='mean', inplace=True)

        ranks = df['mean'].rank(ascending=True, method='min')
        df['aggregate rank'] = ranks

        df['aggregate rank'][df['mean']>(fillna-1)/2] = None
        return df

    def __str__(self):
        df = self.get_ranking()
        return df.to_string()



from dreamtools import D5C2
from nose.tools import assert_almost_equal
import os


def test_d5c2():
    # load data first
    s = D5C2(Ntf=2)  # temporary directory will be created
    s.download_templates()
    filename = s.directory + os.sep + 'templates.txt.gz'
    s.score(filename)
    v = s.get_table().ix[20].Pearson
    assert_almost_equal(v, 0.51295034)

    s.cleanup()



# -*- python -*-
# -*- coding: utf-8 -*-
#
#  This file is part of dreamtools software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.com/dreamtools
#
##############################################################################
"""Module to manage and test the HPN leaderboards

::

    e = hpn.HPNAdmin()
    submissions = e.get_submissions_network()


"""
import os
import time
import copy
import dateutil.parser
import json
import dateutil

from datetime import datetime
import dateutil.parser
from dateutil.relativedelta import *
from dateutil import tz


from dreamtools.core.sageutils import Login
from dreamtools.core.ziptools import ZIP
from scoring import *
from submissions import SubmissionTools

__all__  = ["HPNAdmin"]
# this is an evaluation that can be used for READ/WRITE and tests
test_evaluationID = 1963028

INITIAL_SUBMISSION_STATE = "RECEIVED"

# Those are the real evaluation ID not to be used for testing. Read only
hpn_evaluationIDs = ["1917801", "1917802", "1917803", "1917804", "1917805"]
SC1A_INDEX = 0
SC1B_INDEX = 1
SC2A_INDEX = 2
SC2B_INDEX = 3
SCVIS_INDEX = 4

experimentalDataID = "synxxxx"
insilicoDataID = "synyyyyy"


class HPN(Login, ZIP):
    """Base class for HPN scoring leaderboard classes"""
    def __init__(self, username=None, password=None, client=None):
        super(HPN, self).__init__(client=client)

        self.production = False
        self.subtools = SubmissionTools(client=self.client)

    def json(self, data):
        return json.dumps(data)


class HPNAdmin(HPN):
    """Class to manage the leaderboards

    .. warning::    FOR ADMINISTRATORS ONLY

    You can retrieve the submissions, compute the scores and update the
    leaderboards from here.

    """
    def __init__(self, challengeID=1720047, eid=None, username=None,
            password=None, ec2_key=None, ec2_secret=None, client=None):
        super(HPNAdmin, self).__init__(username=username, password=password,
                client=client)
        self.ec2_key = ec2_key
        self.ec2_secret = ec2_secret
        #self.mode = "latest_scored_or_latest_invalid"
        self.mode = "latest_scored_or_invalid_over_all_weeks"
        self.challengeID = challengeID

        if eid == None:
            print("WARNING: Use HPN DREAM evaluation identifiers  (%s)" % hpn_evaluationIDs)
            #self.evaluation = self.getEvaluation(evaluationID)
            self.evaluationId = hpn_evaluationIDs
        else:
            #self.evaluation = self.getEvaluation(eid)
            if isinstance(eid, list):
                self.evaluationId = eid
            else:
                self.evaluationId = [eid]
        self.profiles = {}

    def getZIPExperimentalData(self):
        filename = self.client.get(experimentalDataID, downloadFile=True)['path']
        z = self.loadZIPFile(filename)
        return z

    def downloadExperimentalMIDAS_main(self):
        """simple function to get the experimental data set from the web and
        extract those with the MIDAS and _main tags"""

        z = self.getZIPExperimentalData()
        for filename in [f.filename for f in z.filelist]:
            if 'MIDAS' in filename and "_main" in filename:
                z.extract(filename)

    def getZIPInSilicoData(self):
        filename = self.client.get(insilicoDataID, downloadFile=True)['path']
        z = self.loadZIPFile(filename)
        return z

    # This should be used only once
    def createEvaluation(self, title="HPN-DREAM Breast Cancer Challenge"):
        evaluation = self.client.createEvaluation(name=title, status="PLANNED", contentSource=self.challengeID)
        self.evaluation = copy.deepcopy(evaluation)
        return evaluation

    def setEvaluationStatus(self, status):
        # This should be used only once
        assert status in ["OPEN", "PLANNED","CLOSED", "COMPLETED", "RECEIVED"]
        evaluation = copy.deepcopy(self.evaluation)
        evaluation['status'] = status
        self.client.restPUT("/evaluation/%s" % evaluation['id'], self.json(evaluation))

    def get_submissions_per_evaluation(self, eid, status, limit=1000,offset=0):
        if status == "OPEN" or status=="INVALID" or status=="SCORED" or  status=="RECEIVED":
            uri = "/evaluation/%s/submission/all?status=%s&limit=%d&offset=%d" % (eid, status, limit, offset)
            return self.client.restGET(uri)
        elif status == "all":
            uri = "/evaluation/%s/submission/all?limit=%d&offset=%d" % (eid,limit, offset)
            return self.client.restGET(uri)
        else:
            raise NotImplementedError

    def tag_submission_as_invalid(self, submission, errormsg):
        eid = submission['id']
        URI = "/evaluation/submission/%s/status" % eid
        status = self.client.getSubmissionStatus(submission)
        if status['status'] in ["OPEN", "RECEIVED"]:
            print("Setting submission (%s) status to INVALID (%s)" % (eid, errormsg))
            status['status'] = "INVALID"
            status['report'] = errormsg
            if self.production:
                self.client.restPUT(URI, self.json(status))
            else:
                print("!!!! to update synapse, set production attribute to True")

    def filter_submissions_by_name(self, submissions, valid_tags=["-Network.zip"], filter=True):
        """Get all submissions that filenames matches one of the tag"""

        if isinstance(valid_tags, str):
            valid_tags = [valid_tags]


        print("Found a total of %s submissions" % submissions['totalNumberOfResults'])

        #print("Keeping only %s submissions with one of the following tags: %s" % (status, valid_tags))
        keeping = []
        # We need to check the validity of the file name. Done here. If a file
        # submitted has not the proper name, then it is ignored. We may want
        # more information about that
        for i, submission in enumerate(submissions['results']):
            handles = [x for x in json.loads(submission["entityBundleJSON"])['fileHandles']]

            if len(handles) == 0 :
                self.tag_submission_as_invalid(submission, "No files were found in the submission")
            else:
                filenames = [x['fileName'] for x in handles]
                found = [filename for tag in valid_tags for filename in filenames if tag in filename]
                if filter == True:
                    if len(found):
                        keeping.append(submission)
                    else:
                        print("submission with following filenames are Invalid: %s" % filenames)
                        print("  extra info: id={} date={}".format(submission['id'], submission['createdOn']))
                        self.tag_submission_as_invalid(submission, "Invalid filename provided %s " % filename)

                else:
                    keeping.append(submission)
        print("Keeping %s submissions" % len(keeping))
        return keeping

    def get_submissions_network(self, eid="1917801", status="all", filter=True):
        """Get only submission with Network.zip in their filenames"""
        print("Getting Network submissions (status %s)" % status)
        submissions = self.get_submissions_per_evaluation(eid, status=status)
        submissions = self.filter_submissions_by_name(submissions,
            ["-Collaborative.zip", "-Network-Code.zip", "-Network.zip", "result.zip"], filter=filter)
        return submissions

    def get_submissions_network_insilico(self, eid="1917802", status="all", filter=True):
        """Get only submission with Network-Insilico.zip in their filenames"""
        print("Getting Network Insilico submissions (status %s)" % status)
        submissions = self.get_submissions_per_evaluation(eid, status=status)
        submissions = self.filter_submissions_by_name(submissions, ["-Network-Insilico.zip", "insilico.zip"], filter=filter)
        return submissions

    def get_submissions_prediction(self, eid="1917803", status="all"):
        """Get only submission with Prediction.zip in their filenames"""
        submissions = self.get_submissions_per_evaluation(eid, status=status)
        submissions = self.filter_submissions_by_name(submissions, ["-Prediction.zip"])
        return submissions

    def get_submissions_prediction_insilico(self, eid="1917804", status="all"):
        """Get only submission with Prediction-Insilico.zip in their filenames"""
        submissions = self.get_submissions_per_evaluation(eid, status=status)
        submissions = self.filter_submissions_by_name(submissions, ["-Prediction-Insilico.zip"])
        return submissions

    def get_submissions_visualization(self, eid="1917805", status="all"):
        """Get only submission with Visualization.zip in their filenames"""
        submissions = self.get_submissions_per_evaluation(eid, status=status)
        submissions = self.filter_submissions_by_name(submissions, ["-Visualization.zip"])
        return submissions

    def summary(self):
        """Print information about submissions"""
        if isinstance(self.evaluationId, list):
            evaluationId = self.evaluationId[0]

        participants = self.client.restGET("/evaluation/%s/participant" % evaluationId)
        print("Evaluation Id=%s" % evaluationId)
        print("There are %s participants who have submitted so far." % participants['totalNumberOfResults'])

        print("------------------------------------------------ SC1A")
        N = len(self.get_submissions_network(status="OPEN"))
        print("There are %s OPEN submissions in SC1A\n." % N)
        N = len(self.get_submissions_network(status="SCORED"))
        print("There are %s SCORED submissions in SC1A.\n" % N)
        N = len(self.get_submissions_network(status="INVALID"))
        print("There are %s INVALID submissions in SC1A.\n" % N)


        print("----------------------------------------------- SC1B")
        N = len(self.get_submissions_network_insilico(status="OPEN"))
        print("There are %s OPEN submissions in SC1B.\n" % N)
        N = len(self.get_submissions_network_insilico(status="SCORED"))
        print("There are %s SCORED submissions in SC1B.\n" % N)
        N = len(self.get_submissions_network_insilico(status="INVALID"))
        print("There are %s INVALID submissions in SC1B.\n" % N)

        print("-------------------------------------------------- SC2A")
        N = len(self.get_submissions_prediction(status="OPEN"))
        print("There are %s OPEN submissions in SC2A.\n" % N)
        N = len(self.get_submissions_prediction(status="SCORED"))
        print("There are %s SCORED submissions in SC2A.\n" % N)
        N = len(self.get_submissions_prediction(status="INVALID"))
        print("There are %s INVALID submissions in SC2A.\n" % N)

        print("---------------------------------------------------- SC2B")
        N = len(self.get_submissions_prediction_insilico(status="OPEN"))
        print("There are %s OPEN submissions in SC2B.\n" % N)
        N = len(self.get_submissions_prediction_insilico(status="SCORED"))
        print("There are %s SCORED submissions in SC2B.\n" % N)
        N = len(self.get_submissions_prediction_insilico(status="INVALID"))
        print("There are %s INVALID submissions in SC2B.\n" % N)

    def update_submission_status(self, submission, scoring, production=False):
        """Update the status of a submission given Scoring instance

        :param int eid: identifier of the submission
        :param scoring: instance of HPNScoring with computed score
        """
        # Update of the submission status given score and status
        self.logger.debug("Updating submission status to {0}".format(scoring.status))
        eid = submission['id']
        submission_status = self.client.getSubmissionStatus(submission)
        submission_status['status'] = scoring.status
        if scoring.score: # otherwise we send a None
            submission_status['score'] = scoring.score
        submission_status['report'] = scoring.report

        print scoring.status, scoring.score, scoring.report
        if scoring.exception:
            assert scoring.status == "INVALID"
            print scoring.exception.value
            submission_status['report'] = scoring.exception.value
        print submission_status
        URI = "/evaluation/submission/%s/status" % eid
        if production == True:
            self.client.restPUT(URI, self.json(submission_status))

    def set_submission_status(self, submission, status="OPEN"):
        eid = submission['id']
        submission_status = self.client.getSubmissionStatus(submission)
        submission_status['status'] = "OPEN"
        URI = "/evaluation/submission/%s/status" % eid
        self.client.restPUT(URI, self.json(submission_status))

    def update_scores_prediction(self, production=False):
        """Not to be used if the notifier is on since score_submission_SC2A
        scores the submissions """
        print("Updating HPN-DREAM submissions related to sub challenge Prediction")
        print("===================================================================")
        print("Get all predictions related to SC2A")
        submissions = self.get_submissions_prediction(status="OPEN")
        submissions2 = self.get_submissions_network(status="RECEIVED")
        submissions = submissions + submissions2
        for i, submission in enumerate(submissions):
            print("\n--------- Scoring submissions %s " % (i+1))
            print("--- Submitter alias: %s" % submission['submitterAlias'])

            self.score_submission_SC2A(submission, 1 ,production=production)

    def update_scores_prediction_insilico(self, production=False):
        """Not to be used if the notifier is on since score_submission_SC2b
        scores the submissions """
        print("Updating HPN-DREAM submissions related to Challenge Prediction Insilico")
        print("=======================================================================")
        print("Get all predictions related to SC2B")
        submissions = self.get_submissions_prediction_insilico(status="OPEN")
        submissions2 = self.get_submissions_network(status="RECEIVED")
        submissions = submissions + submissions2
        for i, submission in enumerate(submissions):
            print("\n--------- Scoring submissions %s " % (i+1))
            print("--- Submitter alias: %s" % submission['submitterAlias'])
            self.score_submission_SC2B(submission, 1, production=production)

    def update_scores_network(self, production=False):
        """Not to be used if the notifier is on since score_submission_SC1A
        scores the submissions """
        print("Updating HPN-DREAM submissions related to sub challenge Network")
        print("===============================================================")
        print("Get all predictions related to Network sub challenge")

        # First, we need to retrieve all submissions and compute the AUC using
        submissions = self.get_submissions_network(status="OPEN")
        submissions2 = self.get_submissions_network(status="RECEIVED")
        submissions = submissions + submissions2

        for i, submission in enumerate(submissions):
            print("\n--------- Scoring submissions %s " % (i+1))
            print("--- Submitter alias: %s" % submission['submitterAlias'])
            self.score_submission_SC1A(submission, 1, production=production)

    def update_scores_network_insilico(self, production=False):
        """Not to be used if the notifier is on since score_submission_SC1B
        scores the submissions """
        print("Updating HPN-DREAM submissions related to sub challenge Network Insilico")
        print("===========================================================================")
        print("Get all predictions related to SC1B")

        # First, we need to retrieve all submissions and compute the AUC using
        # compute_score() method for the entries that are OPEN.
        submissions = self.get_submissions_network_insilico(status="OPEN")
        submissions2 = self.get_submissions_network(status="RECEIVED")
        submissions = submissions + submissions2

        for i, submission in enumerate(submissions):
            print("\n--------- Scoring submissions %s from %s " % (i+1, submission['submitterAlias']))
            self.score_submission_SC1B(submission, 1, production=production)

    def _get_ranking_network(self, submissions):
        print "Using %s submissions to compute the ranking" % len(submissions)
        ranking = HPNScoringNetwork_ranking(client=self.client)
        for i, submission in enumerate(submissions):
            try:
                print("Retrieving status %s " % i)
                teamName = submission['submitterAlias'] + "_id%s"  % i
                status = self.client.getSubmissionStatus(submission)
            except:
                print("could not retrieve teamName or status of submission %s" % submission["entityId"])
                continue
            # otherwise, let us add the auc to the ranking
            try:
                auc = json.loads(status['report'])
                if status['status'] == "SCORED":
                    ranking.add_auc(auc, teamName)
                else:
                    ranking.add_auc(None, teamName)
                # Now, let us keep track of this team name and submission index
                # for updates
            except:
                if status['status'] != "INVALID":
                    print("warning:: could not load report of submission %s-%s"
                        % (submission["entityId"], submission['submitterAlias']))
                ranking.add_auc(None, teamName)
        return ranking

    def _get_ranking_prediction(self, submissions):
        print "Using %s submissions to compute the ranking" % len(submissions)
        ranking = HPNScoringPrediction_ranking(client=self.client)
        for i, submission in enumerate(submissions):
            try:
                teamName = submission['submitterAlias'] + "_id%s"  % i
                status = self.client.getSubmissionStatus(submission)
            except:
                print("could not retrieve teamName or status of submission %s" % submission["entityId"])
                continue
            # otherwise, let us add the auc to the ranking
            try:
                rmse = json.loads(status['report'])
                if status['status'] == "SCORED":
                    ranking.add_rmse(rmse, teamName)
                else:
                    ranking.add_rmse(None, teamName)
                # Now, let us keep track of this team name and submission index
                # for updates
            except:
                print("warning:: could not load report of submission %s-%s"
                    % (submission["entityId"], submission['submitterAlias']))
                ranking.add_rmse(None, teamName)
        return ranking

    def _get_ranking_prediction_insilico(self, submissions):
        print "Using %s submissions to compute the ranking" % len(submissions)
        ranking = HPNScoringPredictionInsilico_ranking(client=self.client)
        for i, submission in enumerate(submissions):
            try:
                teamName = submission['submitterAlias'] + "_id%s"  % i
                status = self.client.getSubmissionStatus(submission)
            except:
                print("could not retrieve teamName or status of submission %s" % submission["entityId"])
                continue
            # otherwise, let us add the auc to the ranking
            try:
                rmse = json.loads(status['report'])
                if status['status'] == "SCORED":
                    ranking.add_rmse(rmse, teamName)
                else:
                    ranking.add_rmse(None, teamName)
            except:
                print("warning:: could not load report of submission %s-%s"
                    % (submission["entityId"], submission['submitterAlias']))
                ranking.add_rmse(None, teamName)
        return ranking

    def _select_submissions(self, submissions):
        submissions = self._clean_multiple_accounts(submissions)
        if self.mode == "latest_scored_or_latest_invalid":
            submissions = self._select_submissions_mode1(submissions)
        elif self.mode == "latest_scored_or_invalid_over_all_weeks":
            submissions = self._select_submissions_mode2(submissions)
        return submissions

    def _select_submissions_mode2(self, submissions):
        # Each week select the latest valid score or invalid submission
        # ignore week 0
        leaderboard = []
        weeks = list(set([s['week'] for s in submissions]))
        if 0 in weeks:
            weeks.remove(0)
        for week in weeks:
            week_submissions = [s for s in submissions if s['week'] == week]
            week_submissions = self._select_submissions_mode1(week_submissions)
            leaderboard.extend(week_submissions)
        return leaderboard

    def _clean_multiple_accounts(self, submissions):
        # known users who have different accounts:
        for sub in submissions:
            if sub['userId'] == '422256':
                sub['userId'] = '1991105'
        return submissions

    def _select_submissions_mode1(self, submissions):
        """keeps only latest scored, latest invalid and open submissions


        ignores "OPEN" and put a warning

        """

        leaderboard = []
        userIds = self._get_participants(submissions)

        # duplicated user id sakev team uses (u'422256' and '1991105')

        print("Select submissions (latest invalid or latest scored) amongst %s"% len(submissions))
        # search maximum version submitted
        for user in set(userIds):
            # need to map the indices of all entries to this user
            scored = [x for x in submissions if x['userId'] == user and x['substatus']['status'] == "SCORED"]
            unscored = [x for x in submissions if x['userId'] == user and x['substatus']['status'] in ["INVALID"]]
            stillopen = [x for x in submissions if x['userId'] == user and x['substatus']['status'] in ["RECEIVED", "OPEN"]]
            if len(stillopen):
                print("WARNING: skipping %s OPEN submissions from user %s " % (len(stillopen), user))
            # latest scored
            latest_scored = [x for x in scored if x['versionNumber'] == max([x['versionNumber'] for x in scored])]
            # latest
            latest_unscored = [x for x in unscored if x['versionNumber'] == max([x['versionNumber'] for x in unscored])]

            # in principle there sould be unique version bu it may happen that
            # someone submit a file (e.g., v1), submit it then delete, add back
            # and submit. The latest has version 1 again also it's 2 different
            # files sugmitted from the same project. In this case, we want the
            # most recent one.
            if len(latest_scored) >= 1:
                leaderboard.append(latest_scored[-1])
            if len(latest_unscored) >= 1:
                leaderboard.append(latest_unscored[-1])
        return leaderboard

    def _get_truncated_report(self, report):
        if len(report)>70:
            report = report[0:70] + "...(truncated)"
        return report

    def _get_table_participants_prediction_insilico(self, submissions,
            startweek=0, endweek=1e6):
        # we can alter these submissions because they should not be used for
        # updating synapse anymore. So we add status together with the
        # submission:
        from numpy import isnan, mean
        submissions = self.subtools.attach_status_to_submissions(submissions)
        submissions = self.subtools.attach_week_to_submissions(submissions, "sc2b")
        submissions = [sub for sub in submissions if sub['week']>startweek and sub['week']<=endweek]

        # We filter the submissions to keep only relevant submissions such as
        # the latest scored or lateste invalid defined by the mode attribute
        submissions = self._select_submissions(submissions)

        ranking = self._get_ranking_prediction_insilico(submissions)
        participants = ranking.participants[:]
        mean_ranks = ranking.get_mean_ranks()
        integer_ranks = ranking.get_integer_ranks()
        zscores = ranking.get_mean_zscores()

        lbText = ""
        from numpy import argsort
        unordered_ranks = [mean_ranks[x] for x in participants]
        indices_ordered_ranks = argsort(unordered_ranks)

        rankCounter = 0
        previousAUC = None
        for index in indices_ordered_ranks:

            this_participant = participants[index]
            submission = submissions[index]

            mean_rank = mean_ranks[this_participant]
            integer_rank = integer_ranks[this_participant]
            zscore = zscores[this_participant]

            rmse = ranking.rmse[index]
            all_rmse = [rmse[k1][k2] for k1 in rmse.keys() for k2 in rmse[k1].keys()]
            all_rmse = [x for x in all_rmse if isnan(x)==False] # exclude nan
            mean_rmse = mean(all_rmse)

            currentAUC = mean_rank
            if currentAUC != previousAUC:
                rankCounter += 1
            previousAUC = currentAUC

            # If there are N files submitted, they all have the same score, so
            # the first one can be taken only
            row = copy.deepcopy(submission)
            userId = submission['userId']
            row['rank'] = rankCounter
            row['meanrank'] = mean_rank

            # We need more information:
            if userId in self.profiles:
                profile = self.profiles[userId]
            else:
                profile = self.client.getUserProfile(userId)
                self.profiles[userId] = copy.deepcopy(profile)

            row['user'] = profile['displayName']
            substatus = submission['substatus']
            row['createdOn'] = submission['createdOn']

            row['score'] = mean_rmse
            row['status'] = substatus['status']
            row['zscore'] = zscore
            row['week'] = submission['week']
            if substatus['status'] == "INVALID":
                if "report" in substatus.keys():
                    row['score'] = self._get_truncated_report(substatus['report'])
                else:
                    row['score'] = "unknown error"


            rowText = "|%(rank)s|%(submitterAlias)s|%(name)s(v%(versionNumber)s)|%(week)s|%(entityId)s|%(score)s|%(zscore)s|%(status)s|%(meanrank)s|" % row
            lbText += "\n" + rowText
        return lbText

    def _get_table_participants_prediction(self, submissions, startweek=0,
            endweek=1e6):
        # we can alter these submissions because they should not be used for
        # updating synapse anymore. So we add status together with the
        # submission:
        from numpy import isnan, mean
        submissions = self.subtools.attach_status_to_submissions(submissions)
        submissions = self.subtools.attach_week_to_submissions(submissions, "sc2a")
        submissions = [sub for sub in submissions if sub['week']>startweek and sub['week']<=endweek]

        # We filter the submissions to keep only relevant submissions such as
        # the latest scored or lateste invalid defined by the mode attribute
        submissions = self._select_submissions(submissions)

        ranking = self._get_ranking_prediction(submissions)
        participants = ranking.participants[:]
        mean_ranks = ranking.get_mean_ranks()
        integer_ranks = ranking.get_integer_ranks()
        zscores = ranking.get_mean_zscores()

        lbText = ""
        from numpy import argsort
        unordered_ranks = [mean_ranks[x] for x in participants]
        indices_ordered_ranks = argsort(unordered_ranks)

        rankCounter = 0
        previousAUC = None
        for index in indices_ordered_ranks:

            this_participant = participants[index]
            submission = submissions[index]

            mean_rank = mean_ranks[this_participant]
            integer_rank = integer_ranks[this_participant]
            zscore = zscores[this_participant]

            rmse = ranking.rmse[index]
            all_rmse = [rmse[k1][k2] for k1 in rmse.keys() for k2 in rmse[k1].keys()]
            all_rmse = [x for x in all_rmse if isnan(x)==False] # exclude nan
            mean_rmse = mean(all_rmse)

            currentAUC = mean_rank
            if currentAUC != previousAUC:
                rankCounter += 1
            previousAUC = currentAUC

            # If there are N files submitted, they all have the same score, so
            # the first one can be taken only
            row = copy.deepcopy(submission)
            userId = submission['userId']
            row['rank'] = rankCounter
            row['meanrank'] = mean_rank

            # We need more information:
            if userId in self.profiles:
                profile = self.profiles[userId]
            else:
                profile = self.client.getUserProfile(userId)
                self.profiles[userId] = copy.deepcopy(profile)

            row['user'] = profile['displayName']
            substatus = submission['substatus']
            row['createdOn'] = submission['createdOn']

            row['score'] = mean_rmse
            row['status'] = substatus['status']
            row['zscore'] = zscore
            row['week'] = submission['week']
            if substatus['status'] == "INVALID":
                if "report" in substatus.keys():
                    row['score'] = self._get_truncated_report(substatus['report'])
                else:
                    row['score'] = "unknown error"


            rowText = "|%(rank)s|%(submitterAlias)s|%(name)s(v%(versionNumber)s)|%(week)s|%(entityId)s|%(score)s|%(zscore)s|%(status)s|%(meanrank)s|" % row
            lbText += "\n" + rowText
        return lbText

    def _get_table_participants_network(self, submissions, startweek=0,
            endweek=1e6):
        # we can alter these submissions because they should not be used for
        # updating synapse anymore. So we add status together with the
        # submission:
        submissions = self.subtools.attach_week_to_submissions(submissions, "sc1a")
        submissions = [sub for sub in submissions if sub['week']>startweek and sub['week']<=endweek]
        submissions = self.subtools.attach_status_to_submissions(submissions)

        # We filter the submissions to keep only relevant submissions such as
        # the latest scored or lateste invalid defined by the mode attribute
        submissions = self._select_submissions(submissions)

        # this is the instance to get all ranking information
        ranking = self._get_ranking_network(submissions)

        participants = ranking.participants[:]
        mean_ranks = ranking.get_mean_ranks()
        integer_ranks = ranking.get_integer_ranks()
        zscores = ranking.get_mean_zscores()

        lbText = ""
        from numpy import argsort
        unordered_ranks = [mean_ranks[x] for x in participants]
        indices_ordered_ranks = argsort(unordered_ranks)

        rankCounter = 0
        previousAUC = None
        for index in indices_ordered_ranks:

            this_participant = participants[index]
            submission = submissions[index]

            mean_rank = mean_ranks[this_participant]
            integer_rank = integer_ranks[this_participant]
            zscore = zscores[this_participant]

            currentAUC = mean_rank
            if currentAUC != previousAUC:
                rankCounter += 1
            previousAUC = currentAUC

            # If there are N files submitted, they all have the same score, so
            # the first one can be taken only
            row = copy.deepcopy(submission)
            userId = submission['userId']
            row['rank'] = rankCounter
            row['meanrank'] = mean_rank

            # We need more information:
            if userId in self.profiles:
                profile = self.profiles[userId]
            else:
                profile = self.client.getUserProfile(userId)
                self.profiles[userId] = copy.deepcopy(profile)

            row['user'] = profile['displayName']
            substatus = submission['substatus']
            row['createdOn'] = submission['createdOn']

            row['score'] = substatus['score']
            row['status'] = substatus['status']
            row['zscore'] = zscore
            row['week'] = submission['week']
            if substatus['status'] == "INVALID":
                if "report" in substatus.keys():
                    row['score'] = self._get_truncated_report(substatus['report'])
                else:
                    row['score'] = "unknown error"

            rowText = "|%(rank)s|%(submitterAlias)s|%(name)s(v%(versionNumber)s)|%(week)s|%(entityId)s|%(score)s|%(zscore)s|%(status)s|%(meanrank)s|" % row
            lbText += "\n" + rowText
        return lbText

    def _get_table_participants_network_insilico(self, submissions, startweek=0,
            endweek=1e6):
        submissions = self.subtools.attach_status_to_submissions(submissions)
        submissions = self.subtools.attach_week_to_submissions(submissions, "sc1b")
        submissions = [sub for sub in submissions if sub['week']>startweek and sub['week']<=endweek]

        # We filter the submissions to keep only INVALID and SCORED submissions
        submissions = self._select_submissions(submissions)

        # need to compute the ranking first...
        aucs = []
        for i, submission in enumerate(submissions):
            try:
                report = json.loads(submission['substatus']['report'])
                score = report['auc']
                # scores(AUC) and zscores are directly proportional except that
                # scores are between 0 and 1 and 1 means perfect and zscores are
                # between 0 and infintie and infinite is perfect. Let us work with
                # AUCS
                aucs.append(score)
            except:
                aucs.append(-1000) # should be enough. A bad score is around zero
        from numpy import argsort
        ranks = argsort(aucs)[::-1] # sort and reverse from best to worst
        lbText = ""

        #should fill the rows in the rank order

        rankCounter = 0
        previousAUC = None

        for rank_index in ranks:
            currentAUC = aucs[rank_index]
            if currentAUC != previousAUC:
                rankCounter += 1
            previousAUC = currentAUC
            submission = submissions[rank_index]
            try:
                report = json.loads(submission['substatus']['report'])
                zscore = report['score']
                auc = report['auc']
            except:
                zscore = "undefined"

            # If there are N files submitted, they all have the same score, so
            # the first one can be taken only
            row = copy.deepcopy(submission)
            userId = submission['userId']

            # We need more information:
            if userId in self.profiles:
                profile = self.profiles[userId]
            else:
                profile = self.client.getUserProfile(userId)
                self.profiles[userId] = copy.deepcopy(profile)

            row['user'] = profile['displayName']
            substatus = submission['substatus']

            row['score'] = auc
            row['zscore'] = zscore
            row['status'] = substatus['status']
            row['rank'] = rankCounter
            row['week'] = submission['week']
            if substatus['status'] == "INVALID":
                if "report" in substatus.keys():
                    row['score'] = self._get_truncated_report(substatus['report'])
                else:
                    row['score'] = "unknown error"

            rowText = "|%(rank)s|%(submitterAlias)s|%(name)s(v%(versionNumber)s)|%(week)s|%(entityId)s|%(score)s|%(status)s|%(zscore)s|" % row

            lbText += "\n" + rowText
        return lbText

    def _update_wiki_leaderboard(self, wikiId, lbText):
        raise ValueError("update the leaderboard manually please.")
        owner = self.client.get(self.challengeID)
        lbWikiPage = self.client.getWiki(owner, wikiId)
        lbWikiPage['markdown'] = lbText
        lbWikiPage = self.client.store(lbWikiPage)

    def _get_participants(self, submissions):
        participants = list(set([s['userId'] for s in submissions]))
        return participants

    def _get_week(self, challenge):
        subs = SubmissionTools(client=self.client)
        now = datetime.now(tz.tzlocal())
        for i in range(1,100000):
            if i not in subs.sc1a_weeks.keys():
                break
            if challenge == "sc1a":
                thisdate = dateutil.parser.parse(subs.sc1a_weeks[i][1])
            elif challenge == "sc1b":
                thisdate = dateutil.parser.parse(subs.sc1b_weeks[i][1])
            elif challenge == "sc2a":
                thisdate = dateutil.parser.parse(subs.sc2a_weeks[i][1])
            elif challenge == "sc2b":
                thisdate = dateutil.parser.parse(subs.sc2b_weeks[i][1])
            else:
                raise NotImplementedError
            print now, thisdate, thisdate<now
            if (thisdate < now) == False:
                return i-1


    def update_leaderboard_network(self, eid="1917801", wikiId=56830,
            production=False, filter=False):
        """ eid the Evaluation queue from which to draw submissions
            wikiId  --  the ID of the wiki to update
            production -- if True then update the wiki leaderboard, otherwise skip this step
            filter -- if True then filter out submission having illegal file names
        """
        submissions = self.get_submissions_network(eid=eid, filter=False)
        print("updating leaderboard (Network)")
        lbText = """#HPN-DREAM8 Network Inference leaderboard\n\nThis is the leaderboard of the [HPN-DREAM8 sub challenge1A](#!Wiki:syn1720047/ENTITY/56213). \n\nLast update: """ +  time.asctime()
        lbText += "\n\n|Rank|Participant|Submission Name (version)|Week|Submission id|Mean AUC|zscore|Status|mean rank|"

        lbText += self._get_table_participants_network(submissions,
                endweek=self._get_week("sc1a"))
        print(lbText)

    def update_leaderboard_network_insilico(self, eid="1917802", wikiId=56850,
            production=False, filter=False):
        submissions = self.get_submissions_network_insilico(eid=eid, filter=False)
        print("updating leaderboard (SC1B)")
        lbText = """#HPN-DREAM8 Network Inference leaderboard\n\nThis is the leaderboard of the [HPN-DREAM8 sub challenge1B](#!Wiki:syn1720047/ENTITY/56213). \n\nLast update: """ +  time.asctime()
        lbText += "\n\n|Rank|Participant|Submission Name (version)|Week|Submission id|AUC|Status|z-score|"
        lbText += self._get_table_participants_network_insilico(submissions,
                endweek=self._get_week("sc1b"))
        print(lbText)

    def update_leaderboard_prediction(self, eid="1917803", wikiId=56831,
            production=False):
        submissions = self.get_submissions_prediction(eid=eid)
        #submissions = self.get_submissions_prediction()
        print("updating leaderboard (SC2A)")
        lbText = """#HPN-DREAM8 Network Inference leaderboard\n\nThis is the leaderboard of the [HPN-DREAM8 sub challenge2A](#!Wiki:syn1720047/ENTITY/56216). \n\nLast update.""" +  time.asctime()
        lbText += "\n\n|Rank|Participant|Submission Name (version)|Week|Submission id|Mean RMSE|zscore(negative is good)|Status|score (mean rank)|"
        lbText += self._get_table_participants_prediction(submissions,
                endweek=self._get_week("sc2a"))
        print(lbText)

    def update_leaderboard_prediction_insilico(self, eid="1917804",
            wikiId=56851, production=False):
        submissions = self.get_submissions_prediction_insilico(eid=eid)
        print("updating leaderboard (SC2B)")
        lbText = """#HPN-DREAM8 Network Inference leaderboard\n\nThis is the leaderboard of the [HPN-DREAM8 sub challenge2A](#!Wiki:syn1720047/ENTITY/56216). \n\nLast update.""" +  time.asctime()
        lbText += "\n\n|Rank|Participant|Submission Name (version)|Week|Submission id|Mean RMSE|zscore(negative is good)|Status|score (mean rank)|"
        lbText += self._get_table_participants_prediction_insilico(submissions,
                endweek=self._get_week("sc2b"))
        print(lbText)

    def update_leaderboard_visualization(self):
        raise NotImplementedError

    def invalid_score(self, report):
        """ Generates an HPNScoring object for an invalid score
        """
        scoring = HPNScoring()
        scoring.status = "INVALID"
        scoring.report = report
        scoring.score = 0
        return(scoring)

    def score_submission_SC1A(self, submission, round_number, production=True):
        """ Scores the given submission and returns its status
        """
        # Get the data corresponding to this submission and compute score
        try:
            filename = self.client.getSubmission(submission, downloadFile=True)['filePath']
        except:
            filename = "dummy"
            self.logger.debug("1st try block")
            scoring = self.invalid_score("Failed to retrieve submitted file.")

        print filename
        try:
            print("scoring"),
            scoring = HPNScoringNetwork(filename=filename, verbose=False,
                client=self.client)
            if scoring.exception:
                raise scoring.exception
            scoring.compute_score()
            if scoring.exception:
                raise ScoringError
        except ScoringError:
            print("caught scoring error. will be reported")
        except Exception, e:
            self.logger.debug("2nd try blocl")
            report = "Error when scoring submission %s (version %s)" % (submission['id'], submission['versionNumber'])
            print(report)
            report += "\nError may be related to : " + type(e).__name__ +  str(e.args)
            scoring = self.invalid_score(report)

        self.update_submission_status(submission, scoring, production=production)
        self.logger.debug("Exit with status {0}".format(scoring.status))
        return (scoring)

    def score_submission_SC1B(self, submission, round_number, production=True):
        """ Scores the given submission and returns its status
        """
        # Get the data corresponding to this submission and compute score
        try:
            filename = self.client.getSubmission(submission, downloadFile=True)['filePath']
        except:
            filename = "dummy"
            self.logger.debug("1st try block")
            scoring = self.invalid_score("Failed to retrieve submitted file.")

        try:
            scoring = HPNScoringNetworkInsilico(filename=filename,
                verbose=False, client=self.client)
            if scoring.exception:
                raise scoring.exception
            score = scoring.compute_score()
            if scoring.exception:
                raise scoring.exception
            auc = scoring.get_roc().compute_auc()
            if scoring.status != None:
                raise ScoringError
        except ScoringError:
            print("caught scoring error. will be reported")
        except Exception,e:
            self.logger.debug("2nd try block")
            report = "Error when scoring submission %s (version %s)" % (submission['id'], submission['versionNumber'])
            print(report)
            report += "\nError may be related to : " + type(e).__name__ +  str(e.args)
            scoring = self.invalid_score(report)

        if scoring.status == None:
            scoring.status = "SCORED"
            scoring.score = auc
            scoring.report = json.dumps({'auc': auc, 'score':score})
        else:
            if scoring.status != None:
                scoring = self.invalid_score("""Failed to compute score in SC1B (unknown
                    error). Please contact the organisers provided the submission Id""")

        self.update_submission_status(submission, scoring, production=production)
        self.logger.debug("Exit with status {0}".format(scoring.status))
        return (scoring)

    def score_submission_SC2A(self, submission, round_number, production=True):
        """ Scores the given submission and returns its status
        """
        # Get the data corresponding to this submission and compute score
        try:
            filename = self.client.getSubmission(submission, downloadFile=True)['filePath']
        except:
            filename = "dummy"
            self.logger.debug("1st try blocl=k")
            scoring = self.invalid_score("Failed to retrieve submitted file.")

        try:
            scoring = HPNScoringPrediction(filename=filename, client=self.client)
            if scoring.exception:
                raise scoring.exception
            scoring.compute_all_rmse()
            if scoring.exception:
                raise scoring.exception
        except ScoringError:
            print("caught scoring error. will be reported")
        except Exception, e:
            self.logger.debug("2nd try block")
            report = "Unknown error when scoring submission %s (version %s)" % (submission['id'], submission['versionNumber'])
            report += "\nError may be related to : " + type(e).__name__ +  str(e.args)
            print(report)
            scoring = self.invalid_score(report)

        if scoring.status == None and len(scoring.rmse['BT20']):
            scoring.report = json.dumps(scoring.rmse)
            scoring.status = "SCORED"
            scoring.score = 1
        else:
            if scoring.status == None:
                scoring = self.invalid_score("""Failed to compute score in SC2A (unknown
                    error). Please contact the organisers provided the submission Id""")

        self.update_submission_status(submission, scoring, production=production)
        self.logger.debug("Exit with status {0}".format(scoring.status))
        return (scoring)

    def score_submission_SC2B(self, submission, round_number, production=True):
        """ Scores the given submission and returns its status
        """
        # Get the data corresponding to this submission and compute score
        try:
            filename = self.client.getSubmission(submission, downloadFile=True)['filePath']
        except:
            filename = "dummy"
            self.logger.debug("1st try block")
            scoring = self.invalid_score("Failed to retrieve submitted file.")

        print filename
        try:
            scoring = HPNScoringPredictionInsilico(filename=filename,
                client=self.client)
            if scoring.exception:
                raise scoring.exception
            scoring.compute_all_rmse()
            if scoring.exception:
                raise scoring.exception
        except ScoringError:
            print("caught scoring error. will be reported")
        except Exception, e:
            self.logger.debug("2nd try block")
            report = "Error when scoring submission %s (version %s)" % (submission['id'], submission['versionNumber'])
            report += "\nError may be related to : " + type(e).__name__ +  str(e.args)
            print(report)
            scoring = self.invalid_score(report)

        if scoring.status == None and len(scoring.rmse['AB1']):
            scoring.report = json.dumps(scoring.rmse)
            scoring.status = "SCORED"
            scoring.score = 1
        else: # is this possible that status !None
            if scoring.status == None:
                scoring = self.invalid_score("""Failed to compute score in SC2B (unknown
                    error). Please contact the organisers provided the submission Id""")


        self.update_submission_status(submission, scoring, production=production)
        self.logger.debug("Exit with status {0}".format(scoring.status))
        return (scoring)

    def score_submission_SCVIS(self, submission, round_number, production=True):
        # "Not yet implemented"
        """ Scores the given submission and returns its status
        """
        # Get the data corresponding to this submission and compute score
        try:
            filename = self.client.getSubmission(submission, downloadFile=True)['filePath']
            scoring = HPNScoring()
            scoring.status = "SCORED"
            scoring.report = ""
            scoring.score = 1
        except:
            filename = "dummy"
            self.logger.debug("1st try block")
            scoring = self.invalid_score("Failed to retrieve submitted file.")
        print filename
        self.update_submission_status(submission, scoring, production=production)
        return (scoring)

    def get_submissions_in_date_range(self, start_date, end_date, sc_index):
        #start_date = dateutil.parser.parse(start_date_string)
        #end_date = dateutil.parser.parse(end_date_string)
        submissions = self.client.getSubmissions(self.evaluationId[sc_index])
        result = {}
        count = 0
        for i, submission in enumerate(submissions):
            count=count+1
            timestamp = dateutil.parser.parse(submission.createdOn)
            if (timestamp>=start_date and timestamp<end_date):
                result[timestamp] = submission
        print("get_submissions_in_date_range: eid="+self.evaluationId[sc_index]+
              ", total count="+str(count)+", # in range="+str(len(result)))
        return (result)




metadata = {
    'sc1a': {"true_synapse_id": "syn1876068"},
}


import os
from dreamtools import dreampath
d8c3path = os.sep.join([dreampath, 'dream8', 'D8C3'])


#gold standard prediction: https://www.synapse.org/#!Synapse:syn1902669 24Mb




# -*- python -*-
#
#  This file is part of dreamtools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.com/dreamtools
#
##############################################################################
"""Module to download final submissions for admin usage only

:status: FINAL

"""
import os
from dreamtools.dream8.D8C1 import submissions
from dreamtools.core.sageutils import Login


class SubmissionsDownloader(Login):
    """Tool to download all official submissions from synapse (zipped files).

    ::

        >>> from dreamtools.dream8.D8C1 import downloads
        >>> d = downloads.SubmissionsDownloader(directory="download")
        >>> d.download_all()


    """
    def __init__(self,  client=None):
        super(SubmissionsDownloader, self).__init__(client=client)
        from dreamtools import configuration as cfg
        mainpath = cfg.user_config_dir
        self.directory = os.sep.join([mainpath, "dream8", "D8C1", "submissions"])

    def _get_location(self, this):
        path = self.directory + os.sep + this
        if os.path.exists(self.directory) == False:
            os.mkdir(self.directory)
        if os.path.exists(path) == False:
            os.mkdir(path)
        return path

    def download_all(self):
        """Download all submissions from the 4 subchallenges"""
        self.download_all_sc1a_final_submissions()
        self.download_all_sc2a_final_submissions()
        self.download_all_sc1b_final_submissions()
        self.download_all_sc2b_final_submissions()

    def download_all_sc1a_final_submissions(self):
        """Download all submissions from the SC1A subchallenge"""
        s = submissions.SC1ASubmissions()
        s.load_submissions()
        for i, sub in enumerate(s.submissions):
            s.client.downloadSubmissionAndFilename(sub, downloadFile=True,
                    downloadLocation=self._get_location("sc1a"))
            print "downloading %s/%s"  % (i+1, len(s.submissions))

    def download_all_sc1b_final_submissions(self):
        """Download all submissions from the SC1B subchallenge

        There is a duplicated filename here. This command fails::

            easydev.swapdict([(x['submitterAlias'],
                    json.loads(x['entityBundleJSON'])['fileHandles'][0]['fileName'])
                                for x in s.submissions])

        The duplicated name is Bing-Network from Bing or Cai team name
        """
        s = submissions.SC1BSubmissions()
        s.load_submissions()
        for i,sub in enumerate(s.submissions):
            s.client.downloadSubmissionAndFilename(sub, downloadFile=True,
                    downloadLocation=self._get_location("sc1b"))
            print "downloading %s/%s"  % (i+1, len(s.submissions))

    def download_all_sc2a_final_submissions(self):
        """Download all submissions from the SC2A subchallenge"""
        s = submissions.SC2ASubmissions()
        s.load_submissions()
        for i,sub in enumerate(s.submissions):
            s.client.downloadSubmissionAndFilename(sub, downloadFile=True,
                    downloadLocation=self._get_location("sc2a"))
            print "downloading %s/%s"  % (i+1, len(s.submissions))

    def download_all_sc2b_final_submissions(self):
        """Download all submissions from the SC2B subchallenge"""
        s = submissions.SC2BSubmissions()
        s.load_submissions()
        for i,sub in enumerate(s.submissions):
            s.client.downloadSubmissionAndFilename(sub, downloadFile=True,
                    downloadLocation=self._get_location("sc2b"))
            print "downloading %s/%s"  % (i+1, len(s.submissions))


class GSDownloader(Login):
    """Factory to download gold standard files

    .. todo:: could be replaced with the dreamtools.core.download.Downloader class
    """
    def __init__(self, client=None):
        super(GSDownloader, self).__init__(client=client)
        from dreamtools import configuration as cfg
        mainpath = cfg.user_config_dir
        self.directory = os.sep.join([mainpath, "dream8", "D8C1"])


    def download_experimental(self):
        self.client.get("syn1920412", downloadLocation=self.directory)







from dreamtools.dream8.D8C1 import ranking

from easydev import get_home
import os
path2data = os.sep.join(['.config', 'dreamtools', 'dream8', 'D8C1', 'submissions'])

def test_sc1a_ranking():
    r = ranking.SC1A_ranking()
    r.append_submission(os.sep.join([get_home(), path2data, "sc1a", 
        "limax-Network.zip"]))
    assert r.get_rank_your_submission() == 2

def test_sc1b_ranking():
    r = ranking.SC1B_ranking()
    r.append_submission(os.sep.join([get_home(), path2data, "sc1b",
        "NMSUSongLab-Network-Insilico.zip"]))
    assert r.get_rank_your_submission() == 1


def test_sc2a_ranking():
    r = ranking.SC2A_ranking()
    r.append_submission(os.sep.join([get_home(), path2data, "sc2a", 
        "ALAK-Prediction.zip"]))
    assert r.get_rank_your_submission() == 1

def test_sc2b_ranking():
    r = ranking.SC2B_ranking()
    r.append_submission(os.sep.join([get_home(), path2data, "sc2b",
        "CGR-Prediction-Insilico.zip"]))
    assert r.get_rank_your_submission() == 1





# -*- python -*-
# -*- coding: utf-8 -*-
#
#  This file is part of dreamtools software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.com/dreamtools
#
############################################################################### -*- python -*-
import copy
import os
import glob
import json

import numpy as np
import pylab
import pandas as pd

import submissions
from dreamtools import Login
from dreamtools.core.ziptools import ZIP

from scoring import HPNScoringPrediction
from scoring import HPNScoringPredictionInsilico
from scoring import HPNScoringNetwork
from scoring import HPNScoringNetworkInsilico
import commons

from dreamtools.dream8.D8C1 import d8c1path
from dreamtools.dream8.D8C1 import scoring

__all__ = ["SC2AggregationPlotting", "SC2A_aggregation", "SC2B_aggregation",
    "SC1AggregationPlotting", "SC1A_aggregation", "SC1B_aggregation"]


class AggregationTools(Login):
    """common class to be used for the aggregation performances

    .. warning:: cannot be used by itself. You should use
    :class:`SC1A_aggregation` for instance.


    """
    def __init__(self, name, client=None):
        assert name in ["SC1A", "SC1B", "SC2A", "SC2B"]
        self.name = name
        super(AggregationTools, self).__init__(client=client)
        self.mode = "mean"

    def load_submissions(self):
        """Load all submissions"""
        if self.name == "SC1A":
            s = submissions.SC1ASubmissions(client=self.client)
        elif self.name == "SC1B":
            s = submissions.SC1BSubmissions(client=self.client)
        elif self.name == "SC2A":
            s = submissions.SC2ASubmissions(client=self.client)
        elif self.name == "SC2B":
            s = submissions.SC2BSubmissions(client=self.client)

        s.load_submissions()
        self.submissions = s.submissions[:]

    def get_best_submissions(self, N, start=0):
        """Return the N best submissions"""

        ranking = [this['ranking'] for this in self.submissions]
        indices = np.argsort(ranking)[start:N]
        subs = [self.submissions[i] for i in indices]
        return subs

    def aggregate_submissions(self, N, start=0):
        """Get the N best submissions and average them

        a method called :meth:`_aggregate` must be defined in the child class

        """
        aggregate = self._aggregate(range(start,N))
        return aggregate

    def aggregate_submissions_random(self, N=10):
        """Aggregate N submissions by picking

        a method called :meth:`_aggregate` must be defined in the child class

        """
        # select N random submissions
        indices = range(0, len(self.df.index))
        pylab.shuffle(indices)
        indices = indices[0:N]
        aggregate = self._aggregate(indices)
        return aggregate

    def get_df_from_submissions(self):
        subs = self.submissions
        df = pd.DataFrame()
        df['submitterAlias'] = [this['submitterAlias'] for this in subs]

        df['userId'] = [this['userId'] for this in subs]
        df['entityId'] = [this['entityId'] for this in subs]
        df['submissionId'] = [this['id'] for this in subs]

        if self.name == "SC1A":
            df['mean_rank'] = [this['ranking'] for this in subs]
            df['mean_auc'] = [this['mean_aucs'] for this in subs]
            df['mean_zscore'] = [this['zscore'] for this in subs]

            df['ranks'] = [this['ranks'] for this in subs]
            df['aucs'] = [this['aucs'] for this in subs]
            df['zscores'] = [this['zscores'] for this in subs]

        if self.name == "SC1B":
            df['auc'] = [this['auc'] for this in subs]
            df['mean_auc'] = [this['auc'] for this in subs]
            df['zscore'] = [this['zscore'] for this in subs]

        if self.name == "SC2A":
            df['mean_rank'] = [this['ranking'] for this in subs]
            df['mean_rmse'] = [this['mean_rmse'] for this in subs]
            df['mean_zscore'] = [this['zscore'] for this in subs]

            df['ranks'] = [this['ranks'] for this in subs]
            df['rmses'] = [this['rmses'] for this in subs]
            df['zscores'] = [this['zscores'] for this in subs]

        if self.name == "SC2B":
            df['mean_rank'] = [this['ranking'] for this in subs]
            df['mean_rmse'] = [this['mean_rmse'] for this in subs]
            df['zscore'] = [this['zscore'] for this in subs]
            df['rmses'] = [this['rmses'] for this in subs]

        try:
            get_filename = lambda sub: json.loads(sub['entityBundleJSON'])['fileHandles'][0]['fileName']
            prefix = os.sep.join([d8c1path, 'submissions', self.name.lower() ])
            df['filename'] = [prefix+ os.sep + get_filename(this) for this in subs]
        except:
            df['filename'] = [this['filename'] for this in subs]

        if self.name == "SC1A" or self.name == "SC2A":
            df.sort(columns="mean_rank", inplace=True)
        elif self.name == "SC1B":
            df.sort(columns="auc", ascending=False, inplace=True)
        elif self.name == "SC2B":
            df.sort(columns='mean_rmse', ascending=True, inplace=True)

        df.reset_index(inplace=True)
        return df

    def _load_submissions_from_synapse(self):
        """To be used only onece to obtain all relevant metadata

        A way to check that ranks are correct is:

        for i in range(0,74):
            ranks = df.set_index("submitterAlias")['ranks'][i]
            print mean([ranks[k1][k2] for k1 in ranks.keys() for k2 in ranks[k1].keys()]) == df['mean_rank'][i]

        """
        self.load_submissions()
        df = self.get_df_from_submissions()
        return df

    def _load_local_submissions(self, directory):
        import glob
        import os
        from easydev import progressbar
        filenames = glob.glob(directory + os.sep + '*zip')
        N = len(filenames)
        print("Found %s zipped file")
        pb = progressbar.progress_bar(N)

        self.submissions = []
        for i, filename in enumerate(filenames):
            sub = {}
            if self.name == 'SC1A':
                raise NotImplementedError

            elif self.name == 'SC1B':
                try:
                    s = scoring.HPNScoringNetworkInsilico(filename, verbose=False)
                    s.compute_score()
                except:
                    print('Skipping %s . Could not read or compute score' %
                            filename)
                    continue
                sub['submitterAlias'] = 'unknown'
                sub['entityId'] = 'unknown'
                sub['id'] = 'unknown'
                sub['auc'] = s.auc
                sub['zscore'] = -1
                sub['filename'] = filename
                sub['userId'] = 'unknonwn'

            elif self.name == 'SC2A':
                raise NotImplementedError


            elif self.name == 'SC2B':
                try:
                    s = scoring.HPNScoringPredictionInsilico(filename, verbose=False)
                    s.compute_all_rmse()
                except:
                    print('Skipping %s . Could not read or compute score' %
                            filename)
                sub['submitterAlias'] = 'unknown'
                sub['entityId'] = 'unknown'
                sub['id'] = 'unknown'
                sub['mean_rmse'] = s.get_mean_rmse()
                sub['zscore'] = -1
                sub['ranking'] = 1
                sub['rmses'] = 1
                

                sub['filename'] = filename
                sub['userId'] = 'unknonwn'


            self.submissions.append(sub)
            pb.animate(i+1,0)

        df = self.get_df_from_submissions()
        if self.name == 'SC2B':
            df['mean_rank'] = df.index


        # We must compute the scores of each team

        #df = self.get_df_from_submissions()
        return df


class SC1AggregationPlotting(object):
    """ABC class plotting common to SC2A_aggregation and SC2B_aggregation."""

    def __init__(self):
        pass

    def plot_aggr_best_score(self, M=None, marker='o', color="r",
                             markersize=6):
        """Plots scores of the aggregation of the best submissions

        The submissionsfor selected are those found in the range 0
        to M. The scores of the individual submissions are also plotted.

        :param m: use the M th best solutions

        .. seealso:: :class:`SC1A_aggregation`

        .. note:: the curve that shows the aggregation of the submissions is
            shifted by +1 on the x-axis since it must have at least 2
            submissions to aggregate.

        .. note:: takes about 250 seconds

        """
        if M is None:
            M = len(self.df.index)
        if M>len(self.df.index):
            M = len(self.df.index)

        mean_aucs = []
        span = range(1, M+1)

        # used by paper module.
        self.results = []
        for i in span:
            aggr = self.aggregate_submissions(i, start=0)
            aggr.compute_score()
            mu = self.compute_grand_mean_auc(aggr.auc)
            mean_aucs.append(mu)
            print i, mu
            self.results.append(aggr.auc)

        self._mean_auc = mean_aucs[:]
        self._best_results = {'x': span}
        # those are the best submitter. Nothing to recompute, can be extracted
        # from the df itself.

        iauc = [self.df.ix[x].mean_auc for x in range(0, M)]

        pylab.clf()
        pylab.plot(span, iauc, marker+color, markersize=markersize,
                   label="AUC (individual submissions)")
        pylab.plot(span, mean_aucs, 'x-',
                   label="{} aggregation (over first N submissions)".format(self.mode))
        pylab.grid(True)
        self._best_results['aggregation'] = mean_aucs
        self._best_results['individual'] = iauc

        pylab.xlabel("N", fontsize=20)
        pylab.ylabel("AUROC", fontsize=20)
        pylab.title("Aggregated AUROC (best case)", fontsize=20)
        yr = pylab.ylim()
        pylab.axis([0.5, M+1, yr[0]-0.05, yr[1]+0.05])
        pylab.ylim([0.35, 0.86])
        pylab.legend(loc="lower left")
        return self._best_results

    def plot_aggr_random(self, N=5, Nmax=10,
                         marker="o", color="r", markersize=6, results=None):
        """
        :param N: repeat N times the aggregation to obtain some errors
        :param Nmax: takes at most N values randomly chosen

        .. image:: sc1a_aggregation_random.png
            :width: 50%

        ::

            >>> from dreamtools.dream8.D8C1 import agregation
            >>> a = aggregation.SC1A_aggregation()
            >>> a.plot_aggr_random(N=100, Nmax=74)

        .. note:: takes about 300s to compute for N=1, Nmax=74 for SC1A
        """
        assert Nmax >= 1
        span = range(1, Nmax+1)
        if results!=None:
            self.results = results.copy()
        else:
            results = np.zeros((N, Nmax))
            for i,n in enumerate(range(0, N)):
                print("Replicate %s" % (i+1) )
                for j, nmax in enumerate(range(1, Nmax+1)):
                    print('----'+str(j))
                    aggr = self.aggregate_submissions_random(nmax)
                    aggr.compute_score()
                    mu = self.compute_grand_mean_auc(aggr.auc)
                    results[i][j] = mu

            self.results = results
        #mean_aucs = results.mean(axis=0)

        self._plot_aggr_random(span, Nmax, markersize=markersize,
                                      marker=marker, color=color)
        return self._random_results

    def _plot_aggr_random(self, span, Nmax, marker='o', color='r', markersize=6):
        # those are the best submitter. Nothing to recompute, can be extracted
        # from the df itself.
        iauc = [self.df.ix[x].mean_auc for x in range(0, Nmax)]

        pylab.clf()
        pylab.plot([x for x in span], iauc, marker+color, markersize=markersize,
                   label="AUC (individual submissions)".format(self.mode))
        pylab.grid(True)
        #pylab.plot()
        pylab.xlabel("N", fontsize=20)
        pylab.ylabel("AUROC", fontsize=20)
        pylab.title("Aggregated AUROC (random case)", fontsize=20)

        pylab.errorbar(span, self.results.mean(axis=0), self.results.std(axis=0),
                       label="{} aggregation (over N submissions)".format(self.mode))
        pylab.legend(loc="lower left")

        self._random_results = {}
        self._random_results['x'] = span
        self._random_results['individual'] = iauc
        self._random_results['aggregation_mean'] = list(self.results.mean(axis=0))
        self._random_results['aggregation_std'] = list(self.results.std(axis=0))
        self._random_results['aggregation_all'] = [list(x) for x in self.results]

        xmax = pylab.xlim()[1]
        pylab.ylim([0.35, 0.86])
        pylab.xlim(0.5, xmax)

    def compute_grand_mean_auc(self, data):
        if isinstance(data, dict):
            return np.mean([data[k1][k2] for k1 in data.keys()
                for k2 in data[k1].keys() if k2
                in self.valid_ligands_final[k1]])
        else:
            return data


class SC2AggregationPlotting(object):
    """ABC class plotting common to SC2A_aggregation and SC2B_aggregation."""
    def __init__(self):
        pass

    def plot_aggr_best_score(self, N=2):
        """plots aggregation using best N submissions

        .. seealso:: :class:`SC2A_aggregation`

        """
        assert N>=2
        if N > len(self.df.index):
            N = len(self.df.index)
        mean_rmses = []

        span = range(1, N+1)
        for i in span:
            aggr = self.aggregate_submissions(i)
            aggr.compute_all_rmse()
            rmse = self._get_mean_rmse(aggr.rmse)
            mean_rmses.append(rmse)
            print i, rmse

        iauc = [self.df.ix[x].mean_rmse for x in range(0,N)]

        pylab.clf()
        pylab.plot(span, mean_rmses, 'x-', label="first N aggregation")
        pylab.grid(True)

        newspan = [x for x in span]

        pylab.plot(newspan, iauc, 'or', label="individual mean RMSE")
        pylab.xlabel("N", fontsize=20)
        pylab.ylabel("RMSE", fontsize=20)
        pylab.title("RMSE for the first N best submissions", fontsize=20)
        #yr = pylab.ylim()
        #pylab.axis([1, , start+N+1, yr[0],yr[1]])
        pylab.legend(loc="upper left")
        return iauc

    def plot_aggr_random(self, N=5, Nmax=14):
        """plots aggregation using N random submissions

        .. seealso:: :class:`SC2A_aggregation`

        """
        mean_aucs = []

        assert Nmax>=1
        span = range(1, Nmax+1)
        results = np.zeros((N, Nmax))
        for i,n in enumerate(range(0,N)):
            print("Replicate %s" %i )
            for j,nmax in enumerate(range(1, Nmax+1)):
                aggr = self.aggregate_submissions_random(nmax)
                aggr.compute_all_rmse()
                results[i][j] = self._get_mean_rmse(aggr.rmse)
        self.results = results

        mean_aucs = results.mean(axis=0)

        iauc = [self.df.ix[x].mean_rmse for x in range(0,Nmax)]
        #newspan = [x+1.5 for x in range(start,start+Nmax)]

        pylab.clf()
        self.results = results

        pylab.plot(span, mean_aucs, 'x-', label="Random aggregation ({}) for each N".format(N))
        pylab.grid(True)
        pylab.plot([x for x in span], iauc, 'or', label="individual mean RMSE")

        pylab.xlabel("N", fontsize=20)
        pylab.ylabel("{} RMSE".format(self.mode), fontsize=20)
        pylab.title("{} RMSE using N random submissions".format(self.mode), fontsize=20)
        pylab.errorbar(range(1,Nmax+1), results.mean(axis=0), results.std(axis=0))
        pylab.legend(loc="upper left")
        return iauc


class SC1A_aggregation(AggregationTools, SC1AggregationPlotting):
    """Investigating the aggregation over several teams.

    ::

        >>> from dreamtools.dream8.D8C1 import aggregation
        >>> a = aggregation.SC1A_aggregation()
        >>> a.plot_aggr_random(N=100, Nmax=74)

    By default, uses the submissions from the challenge itself (up to week 9)

    You need to download the file before hand:


    2 regimes ignored while doing the scoring/aggregation.

    """
    valid_ligands_final = commons.valid_ligands_final
    def __init__(self, best=2, client=None, local_submissions=False):
        """

        :param best: default to 2
        :param client: an existing synapse client
        :param submissions: list of submissions already downloaded. Otherwise
            reload all of them
        :param startweek: default is begining of the challenge (week 1)
        :param endweek: default is end of the challenge (week 9)

        """
        super(SC1A_aggregation, self).__init__(name="SC1A", client=client)
        self.best = 2
        self.startweek = 1
        self.endweek = 9

        self._individuals = {}
        self.directory = os.sep.join([d8c1path, 'submissions', 'sc1a'])

        if local_submissions is False:
            self.df = self._load_submissions_from_synapse()
        else:
            self.df = self._load_local_submissions(local_submissions)


        scoring = HPNScoringNetwork()
        self.true_descendants = copy.deepcopy(scoring.true_descendants)

    def remove_correlated_submissions(self):
        teams = ['AHAT', 'NIPL', 'T4', 'Taylor Swift', 'Hatric',
                'ScreamingGoats', 'dftt', 'bdalab']

        self.submissions = [sub for sub in self.submissions if
            sub['submitterAlias'] not in teams]

        self.df = self.get_df_from_submissions()

    def _get_seed_aggregate(self, index):
        if index in self._individuals.keys():
            aggregate = copy.deepcopy(self._individuals[index])
        else:
            filename = self.df.ix[index].filename
            aggregate = HPNScoringNetwork(filename=filename,
                                          true_descendants=self.true_descendants)
            self._individuals[index] = copy.deepcopy(aggregate)
        return aggregate

    def _aggregate(self, subs):
        # let us pick up one submissions and score it just to get the data
        # structure. Could be saved once for all ?

        aggregate = self._get_seed_aggregate(subs[0])
        edge_scores = {'BT20':{}, 'BT549':{}, 'MCF7':{}, 'UACC812':{}}
        for c in aggregate.edge_scores.keys():
            for l in aggregate.edge_scores[c].keys():
                edge_scores[c][l] = [aggregate.edge_scores[c][l]]

        #N = len(subs)
        if len(subs)>1:
            for sub in subs[1:]:
                if sub in self._individuals.keys():
                    individual = copy.deepcopy(self._individuals[sub])
                else:
                    filename =self.df.ix[sub].filename
                    individual = HPNScoringNetwork(filename=filename,
                        true_descendants=self.true_descendants)
                    self._individuals[sub] = copy.deepcopy(individual)
                for c in aggregate.edge_scores.keys():
                    for l in aggregate.edge_scores[c].keys():
                        #aggregate.edge_scores[c][l] += individual.edge_scores[c][l]
                        edge_scores[c][l].append(individual.edge_scores[c][l])

                        if edge_scores[c][l][0].max()>1:
                            print c,l,edge_scores[c][l][0].max()

        for c in aggregate.edge_scores.keys():
            for l in aggregate.edge_scores[c].keys():
                if self.mode == "mean":
                    #aggregate.edge_scores[c][l] /= float(N)
                    aggregate.edge_scores[c][l] = np.mean(edge_scores[c][l], axis=0)
                elif self.mode == "median":
                    aggregate.edge_scores[c][l] = np.median(edge_scores[c][l], axis=0)
        self._edge_scores = edge_scores.copy()
        return aggregate

    def plot_aggregate_edge_rank(self, N=None, ss=None, method="min"):
        """

        See paper for the vectors of aggregsation (best weighted scores)
        and submissions AUCs

        Then chose a method and loop over this function::

            auc_min = []
            for i in range(0,74):
                es = s.plot_aggregate_edge_rank(i, ss=ss)
                print es
                auc_min.append(es)
            plot(auc, label="min")

        with 66 submissions, and method=min we reach 84%
        """

        # first does not really work
        assert method in ["average", "max", "min", "first"]

        # first, we need to load all edge scores.
        if ss is None:
            print("loading a scoring function")
            filename = os.sep.join([d8c1path, 'submissions', 'sc1a', 'DC_GFP-Network.zip'])
            ss = HPNScoringNetwork(filename, true_descendants=self.true_descendants)
        else:
            pass

        if N is None:
            N = len(self.submissions)

        print("Loading all edge scores")
        for sub in range(0, N):
            if sub in self._individuals.keys():
                pass
            else:
                filename = self.df.ix[sub].filename
                individual = HPNScoringNetwork(filename=filename,
                                               true_descendants=self.true_descendants)
                self._individuals[sub] = copy.deepcopy(individual)

        print("Extracting the edge ranks")
        edge_scores = {'BT20':{}, 'BT549':{}, 'MCF7':{}, 'UACC812':{}}
        for sub in range(0, N):
            for c in self.valid_ligands_final.keys():
                for l in self.valid_ligands_final[c]:
                    data = self._individuals[sub].edge_scores[c][l].copy()
                    shape = data.shape
                    ts = pd.TimeSeries(data.flatten())
                    ranks = ts.rank(ascending=True, method=method).as_matrix().reshape(shape)
                    try:
                        edge_scores[c][l].append(ranks)
                    except:
                        edge_scores[c][l] = [ranks]

        print("averaging")
        for c in edge_scores.keys():
            for l in edge_scores[c].keys():
                if self.mode == "mean":
                    #aggregate.edge_scores[c][l] /= float(N)
                    edge_scores[c][l] = np.mean(edge_scores[c][l], axis=0)
                elif self.mode == "median":
                    edge_scores[c][l] = np.median(edge_scores[c][l], axis=0)

        ss.descendancy_matrices = dict([(x,{}) for x in ss.valid_cellLines])
        for c in edge_scores.keys():
            for l in edge_scores[c].keys():
                ss.edge_scores[c][l] = edge_scores[c][l].copy()
        ss.compute_all_aucs()
        return ss.get_auc_final_scoring(), ss


class SC1B_aggregation(AggregationTools, SC1AggregationPlotting):
    """Investigating the aggregation over several teams.

    ::

        >>> from dreamtools.dream8.D8C1 import aggregation
        >>> a = aggregation.SC1B_aggregation()
        >>> a.plot_aggr_random(N=100, Nmax=5)

    By default, uses the submissions from the challenge itself (up to week 9)


    """
    def __init__(self, best=2, client=None, local_submissions=False):
        """

        :param best:
        :param client: an existing synapse client
        :param submissions: list of submissions already downloaded. Otherwise
            reload all of them
        :param startweek: default is begining of the challenge (week 1)
        :param endweek: default is end of the challenge (week 9)

        """
        super(SC1B_aggregation, self).__init__(name="SC1B", client=client)
        self.best = 2

        self._individuals = {}

        self.directory = os.sep.join([d8c1path, 'submissions', 'sc1b'])

        if local_submissions is False:
            self.df = self._load_submissions_from_synapse()
        else:
            self.df = self._load_local_submissions(local_submissions)

    def remove_correlated_submissions(self):
        # same  as in SC1A actually...
        teams = ['AHAT', 'NIPL', 'T4', 'Taylor Swift', 'Hatric',
                'ScreamingGoats', 'dftt', 'bdalab']

        self.submissions = [sub for sub in self.submissions if
            sub['submitterAlias'] not in teams]
        self.df = self.get_df_from_submissions()

    def _get_seed_aggregate(self, index):
        if index in self._individuals.keys():
            aggregate = copy.deepcopy(self._individuals[index])
        else:
            filename = self.df.ix[index].filename
            aggregate = HPNScoringNetworkInsilico(filename=filename)
            self._individuals[index] = copy.deepcopy(aggregate)
        return aggregate

    def _aggregate(self, subs):
        # let us pick up the first submissions and score it just to get the data
        # structure.
        aggregate = self._get_seed_aggregate(subs[0])
        user_graph = []
        user_graph.append(aggregate.user_graph)

        if len(subs)>1:
            for sub in subs[1:]:
                if sub in self._individuals.keys():
                    individual = copy.deepcopy(self._individuals[sub])
                else:
                    filename =self.df.ix[sub].filename
                    individual = HPNScoringNetworkInsilico(filename=filename)
                    self._individuals[sub] = copy.deepcopy(individual)
                user_graph.append(individual.user_graph)
        if self.mode == "mean":
            aggregate.user_graph = np.mean(np.array(user_graph), axis=0)
        elif self.mode == "median":
            aggregate.user_graph = np.median(np.array(user_graph), axis=0)
        return aggregate

    def _get_mean_auc2(self, aggr):
        # nothing special is done here. This is just for
        #the plot_aggr_best_score in sc1a and sc1b to be identical
        return aggr.get_auc()


class SC2A_aggregation(AggregationTools, SC2AggregationPlotting):
    """Investigating the aggregation over several teams.

    ::

        >>> from dreamtools.dream8.D8C1 import aggregation
        >>> a = aggregation.SC2A_aggregation()
        >>> a.plot_aggr_random(N=100, Nmax=14)

    By default, uses the submissions from the challenge itself (up to week 9)


    """
    def __init__(self, client=None, local_submissions=False):
        """.. rubric:: constructor

        :param best:
        :param client: an existing synapse client
        :param submissions: list of submissions already downloaded. Otherwise
            reload all of them

        """
        super(SC2A_aggregation, self).__init__(name="SC2A", client=client)

        self._individuals = {}

        self.directory = os.sep.join([d8c1path, 'submissions', 'sc2a'])
        if local_submissions is False:
            self.df = self._load_submissions_from_synapse()
        else:
            self.df = self._load_local_submissions(local_submissions)

    def _get_seed_aggregate(self, index):
        if index in self._individuals.keys():
            aggregate = copy.deepcopy(self._individuals[index])
        else:
            filename = self.df.ix[index].filename
            aggregate = HPNScoringPrediction(filename=filename)
            self._individuals[index] = copy.deepcopy(aggregate)
        return aggregate

    def _aggregate(self, subs):
        # let us pick up the first submissions and score it just to get the data
        # structure.

        aggregate = self._get_seed_aggregate(subs[0])

        # store all data inside lists to be able to compute median
        values = aggregate.user_prediction.copy()
        for c in aggregate.user_prediction.keys():
            for l in aggregate.user_prediction[c].keys():
                if l not in aggregate.phosphos_to_exclude[c]:
                    for k in aggregate.user_prediction[c][l].keys():
                        values[c][l][k] = [aggregate.user_prediction[c][l][k]]

        if len(subs)>1:
            for sub in subs[1:]:
                if sub in self._individuals.keys():
                    individual = copy.deepcopy(self._individuals[sub])
                else:
                    filename = self.df.ix[sub].filename
                    individual = HPNScoringPrediction(filename=filename)
                    self._individuals[sub] = copy.deepcopy(individual)
                for c in aggregate.user_prediction.keys():
                    for l in aggregate.user_prediction[c].keys():
                        if l not in aggregate.phosphos_to_exclude[c]:
                            for k in aggregate.user_prediction[c][l].keys():
                                #data1 = aggregate.user_prediction[c][l][k]
                                data2 = individual.user_prediction[c][l][k]
                                #aggregate.user_prediction[c][l][k] = [x+y for x,y in zip(data1, data2)]
                                values[c][l][k].append(data2)

        for c in aggregate.user_prediction.keys():
            for l in aggregate.user_prediction[c].keys():
                if l not in aggregate.phosphos_to_exclude[c]:
                    for k in aggregate.user_prediction[c][l].keys():

                        if self.mode == "mean":
                            aggregate.user_prediction[c][l][k] = np.mean(values[c][l][k], axis=0)
                        elif self.mode=="median":
                            aggregate.user_prediction[c][l][k] = np.median(values[c][l][k], axis=0)
        return aggregate

    def _get_mean_rmse(self, data):
        d = data
        return np.nanmean([d[k1][k2] for k1 in d.keys() for k2 in d[k1].keys()])



class SC2B_aggregation(AggregationTools, SC2AggregationPlotting):
    """Investigating the aggregation over several teams.

    ::

        >>> from dreamtools.dream8.D8C1 import aggregation
        >>> a = aggregation.SC2B_aggregation()
        >>> a.plot_aggr_random(N=100, Nmax=5)

    By default, uses the submissions from the challenge itself (up to week 9)




    """
    def __init__(self, client=None, local_submissions=False):
        """

        :param client: an existing synapse client

        """
        super(SC2B_aggregation, self).__init__(name="SC2B", client=client)
        self._individuals = {}
        self.directory = os.sep.join([d8c1path, 'submissions', 'sc2b'])

        if local_submissions is False:
            self.df = self._load_submissions_from_synapse()
        else:
            self.df = self._load_local_submissions(local_submissions)

    def _get_seed_aggregate(self, index):
        if index in self._individuals.keys():
            aggregate = copy.deepcopy(self._individuals[index])
        else:
            filename = self.df.ix[index].filename
            aggregate = HPNScoringPredictionInsilico(filename=filename)
            self._individuals[index] = copy.deepcopy(aggregate)
        return aggregate

    def _aggregate(self, subs):
        # let us pick up the first submissions and score it just to get the data
        # structure.
        aggregate = self._get_seed_aggregate(subs[0])

        # store all data inside lists to be able to compute median
        values = aggregate.user_prediction.copy()
        for c in aggregate.user_prediction.keys():
            for l in aggregate.user_prediction[c].keys():
                for k in aggregate.user_prediction[c][l].keys():
                    values[c][l][k] = [aggregate.user_prediction[c][l][k]]

        if len(subs)>1:
            for sub in subs[1:]:
                if sub in self._individuals.keys():
                    individual = copy.deepcopy(self._individuals[sub])
                else:
                    filename = self.df.ix[sub].filename
                    individual = HPNScoringPredictionInsilico(filename=filename)
                    self._individuals[sub] = copy.deepcopy(individual)
                for c in aggregate.user_prediction.keys():
                    for l in aggregate.user_prediction[c].keys():
                        for k in aggregate.user_prediction[c][l].keys():
                            #data1 = aggregate.user_prediction[c][l][k]
                            data2 = individual.user_prediction[c][l][k]
                            #aggregate.user_prediction[c][l][k] = [x+y for x,y in zip(data1, data2)]
                            values[c][l][k].append(data2)

        # now compute the mean
        for c in aggregate.user_prediction.keys():
            for l in aggregate.user_prediction[c].keys():
                for k in aggregate.user_prediction[c][l].keys():
                    #data = aggregate.user_prediction[c][l][k]
                    #aggregate.user_prediction[c][l][k] = [x/float(N) for x in data]
                    if self.mode == "mean":
                        aggregate.user_prediction[c][l][k] = np.mean(values[c][l][k], axis=0)
                    elif self.mode=="median":
                        aggregate.user_prediction[c][l][k] = np.median(values[c][l][k], axis=0)

        return aggregate

    def _get_mean_rmse(self, data):
        d = data
        return np.mean([d[k1][k2] for k1 in d.keys() for k2 in d[k1].keys() if
            np.isnan(d[k1][k2])==False])


class GenerateSC1AMatrixForMatlabAggregation(Login, ZIP):
    """

    This class expects to find the 74 zipped submissions in
    submission/sc1a directory. If you don't have this directory, please
    use :class:`dreamtools.dream8hpn.downloads.SubmissionsDownloader` class.


    """
    def __init__(self, client=None, edge_scores={}, species={}):
        super(GenerateSC1AMatrixForMatlabAggregation, self).__init__(client=client)

        self.directory = os.sep.join([d8c1path, 'submissions', 'sc1a'])

        self.edge_scores = edge_scores.copy()
        self.species = species.copy()
        self._set_mapping()

        if len(self.edge_scores) == 0 or len(species)==0:
            self.loaddata()

        self.cellLines = ["BT20", "BT549", "MCF7", "UACC812"]
        self.stimuli = ['IGF1', 'PBS', 'Serum', 'NRG1', 'Insulin', 'HGF', 'EGF', 'FGF1']

    def loaddata(self):
        """Team name in the filename are not alwasy the official team names as
        they appear on synapse.

        """
        if os.path.exists(self.directory)==False:
            raise ValueError("directory %s does not exists" % self.directory)
        filenames = glob.glob(self.directory+os.sep+"*zip")
        if len(filenames)!=74:
            raise ValueError("Expected 74 submissions in SC1A final \
submissions.  Found %s" % len(filenames))


        #true_desc_filename = os.sep.join([d8c1path, 'gs', 'TrueDescVectors.zip'])

        for filename in filenames:
            team_name = self.mapping[filename.split("/")[2]]
            individual = HPNScoringNetwork(filename=filename,
                                           true_descendants=self.true_descendants,
                    skip_true=True)
            self.edge_scores[team_name] = individual.edge_scores.copy()
        print("all edge scores available in edge_scores dictionary")
        self.species = individual.species.copy()


    def create_single_dataframe(self, cell, stimulus):

        # first get the names of teach value in the matrices that will be read
        indices = [x+"/"+y for x in self.species[cell] for y in self.species[cell]]

        # each team for a given cell and stimuli has a NxN matrix of weights


        data = [(team, self.edge_scores[team][cell][stimulus].flatten())
            for team in self.edge_scores.keys()]

        df = pd.DataFrame(dict(data), index=indices)
        return df

    def save_all_df(self, threshold=0.5):
        for cell in self.cellLines:
            for s in self.stimuli:
                filename = "%s_%s_agg.csv" % (cell, s)
                print("saving %s" % filename)
                df = self.create_single_dataframe(cell, s)
                df[df<threshold] = -1
                df[df>=threshold] = 1
                df.to_csv(filename)


    def _set_mapping(self):
        """The mapping is harcoded bt can be created as follows:

        s = submissions.SC1ASubmissions()
        easydev.swapdict(dict([(x['submitterAlias'],
            json.loads(x['entityBundleJSON'])['fileHandles'][0]['fileName'])
            for x in s.submissions]))
        """
        self.mapping = {u'ABCD-Network.zip': u'ABCD',
                        u'AHAT-Network.zip': u'AHAT',
                        u'ALAK-Network.zip': u'ALAK',
                        u'AUTO-Network.zip': u'Auto',
                        u'AlSoTi-Network.zip': u'AlSoTi',
                        u'Alphabet-Network.zip': u'Alphabet',
                        u'BIGH-Network.zip': u'BIGH',
                        u'BNet-Network.zip': u'BNet',
                        u'BiGCaT-Network.zip': u'BiGCaT',
                        u'Bing-Network.zip': u'Bing',
                        u'Boston-Network.zip': u'Boston',
                        u'CGR-Network.zip': u'CGR',
                        u'CMK-Network.zip': u'CMK4',
                        u'Cai-Network.zip': u'Cai',
                        u'Cassis-Network.zip': u'Cassis',
                        u'ChaosLab-Network.zip': u'ChaosLab',
                        u'D3-Network.zip': u'D3',
                        u'DC_GFP-Network.zip': u'DC_TDC',
                        u'DynamoBios-Network.zip': u'Dynamo Bios',
                        u'FUBerlin-Network.zip': u'FUBerlin',
                        u'Firefly-Network.zip': u'Firefly',
                        u'Freya-Network.zip': u'Freya',
                        u'Frs5pB-Network.zip': u'Frs5pB',
                        u'GoatTower-Network.zip': u'Goat Tower',
                        u'Gupta3-Network.zip': u'Gupta',
                        u'HD_Systems6-Network.zip': u'HD_Systems',
                        u'HIBY-Network.zip': u'HIBY',
                        u'Hatric-Network.zip': u'Hatric',
                        u'HeelsDream-Network.zip': u'HeelsDream',
                        u'Ingenuity-Network.zip': u'Ingenuity',
                        u'JCheng-Network.zip': u'JCheng',
                        u'JKLab-Network.zip': u'JKLab',
                        u'JSE-Network.zip': u'JSE',
                        u'KALEH-Network.zip': u'KALEH',
                        u'KTLAB-Network.zip': u'KTLAB',
                        u'MA5-Network.zip': u'DoNET',
                        u'ML_BHK_KG-Network.zip': u'ML_BHK_KG',
                        u'Morpheus-Network.zip': u'Morpheus',
                        u'NIPL-Network.zip': u'NIPL',
                        u'NMSUSongLab-Network.zip': u'NMSUSongLab',
                        u'Netzwerk-Network.zip': u'Netzwerk',
                        u'Pitt.transmed-Network.zip': u'Pitt.transmed',
                        u'Platinum-Network.zip': u'Platinum',
                        u'Reptar1-Network.zip': u'Reptar',
                        u'SBIT-Network.zip': u'SBIT',
                        u'SBI_Lab-Network.zip': u'SBI_Lab',
                        u'SENSE-Network.zip': u'SENSE',
                        u'SFTTRW-Network.zip': u'SFTTRW',
                        u'SUNQR-Network.zip': u'SUNQR',
                        u'ScreamingGoats-Network.zip': u'ScreamingGoats',
                        u'Singularity-Network.zip': u'Singularity',
                        u'StuartLab-Network.zip': u'StuartLab',
                        u'StuartLabANOVA-Network.zip': u'StuartLabANOVA',
                        u'T4-Network.zip': u'T4',
                        u'T8-Network.zip': u'T8',
                        u'TaylorSwift-Network.zip': u'Taylor Swift',
                        u'Tongji-Network.zip': u'tongji team',
                        u'Try1-Network.zip': u'TEST-1roni',
                        u'WH-Network.zip': u'WH',
                        u'WelchsLab-Network.zip': u'WelchsLab',
                        u'Zhangroup-Network.zip': u'Zhangroup',
                        u'amss1012-Network.zip': u'amss1012',
                        u'bacbddepn-Network.zip': u'bacbddepn',
                        u'bdalab-Network.zip': u'bdalab',
                        u'dftt-Network.zip': u'dftt',
                        u'guanlab18-Network.zip': u'GuanLab',
                        u'hibiscus-Network.zip': u'hibiscus',
                        u'limax-Network.zip': u'limax',
                        u'lpNet-Network.zip': u'lpNet',
                        u'result.zip': u'ICHING',
                        u'sakev-Network.zip': u'sakev',
                        u'sannio2-Network.zip': u'sannio2',
                        u'sfntt-Network.zip': u'sfntt',
                        u'slugly-Network.zip': u'slugly'}


class GenerateSC1BMatrixForMatlabAggregation(Login, ZIP):
    """

    This class expects to find the 65 zipped submissions in
    hpndream8_downloads/sc1a directory. If you don't have this directory, please
    use :class:`dreamtools.dream8hpn.downloads.SubmissionsDownloader` class.



    """
    def __init__(self,  client=None):
        super(GenerateSC1BMatrixForMatlabAggregation, self).__init__(client=client)
        self.directory = os.sep.join([d8c1path, 'submissions', 'sc1b'])
        self._set_mapping()
        self.user_graphs = {}

    def loaddata(self):
        """Team name in the filename are not alwasy the official team names as
        they appear on synapse.

        """
        if os.path.exists(self.directory)==False:
            raise ValueError("directory %s does not exists" % self.directory)
        filenames = glob.glob(self.directory+os.sep+"*zip")
        if len(filenames)!=65:
            raise ValueError("Expected 65 submissions in SC1A final \
submissions.  Found %s" % len(filenames))

        true_desc_filename = os.sep.join([d8c1path, "gs", "TrueGraph.csv"])

        for filename in filenames:
            team_name = self.mapping[filename.split("/")[2]]
            individual = HPNScoringNetworkInsilico(filename=filename,
                    goldstandard=true_desc_filename)
            self.user_graphs[team_name] = individual.user_graph.copy()
        print("all edge scores available in user_graph dictionary")

    def create_single_dataframe(self, data):

        labels = ["AB"+str(i) for i in range(1,21)]
        indices = [x+"/"+y for x in labels for y in labels]

        df = pd.DataFrame(data.flatten(),
                index=indices)
        return df

    def save_all_network(self, threshold=0.5):
        filename = "sc1b_user_graph_threshold_%s_agg.csv" % (str(threshold).replace(".", "dot"))
        dfs = pd.DataFrame()
        for team,data in self.user_graphs.iteritems():
            df = self.create_single_dataframe(data)
            print team
            dfs[team] = df.iloc[:,0]
        dfs[dfs<threshold] = -1
        dfs[dfs>=threshold] = 1
        dfs.to_csv(filename)


    def _set_mapping(self):
        """The mapping is harcoded bt can be created as follows:

        s = submissions.SC1BSubmissions()
        s.load_submissions()
        d = dict([(x['submitterAlias'],
            json.loads(x['entityBundleJSON'])['fileHandles'][0]['fileName'])
            for x in s.submissions])
        d['Cai'] = 'Bing(Cai)-Network-Insilico.zip'
        mapping = easydev.swapdict(d)
        """
        self.mapping = {u'ABCD-Network-Insilico.zip': u'ABCD',
             u'AHAT-Network-Insilico.zip': u'AHAT',
             u'ALAK-Network-Insilico.zip': u'ALAK',
             u'AlSoTi-Network-Insilico.zip': u'AlSoTi',
             u'Alphabet-Network-Insilico.zip': u'Alphabet',
             u'Antipodes-Network-Insilico.zip': u'Antipodes',
             u'BNet-Network-Insilico.zip': u'BNet',
             u'BiGCaT-Network-Insilico.zip': u'BiGCaT',
             'Bing(Cai)-Network-Insilico.zip': u'Cai',
             u'Bing-Network-Insilico.zip': u'Bing',
             u'Boston-Network-Insilico.zip': u'Boston',
             u'CGR-Network-Insilico.zip': u'CGR',
             u'CMK-Network-Insilico.zip': u'CMK',
             u'Cassis-Network-Insilico.zip': u'Cassis',
             u'ChaosLab-Network-Insilico.zip': u'ChaosLab',
             u'D3-Network-Insilico.zip': u'D3',
             u'DC_GFInS-Network-Insilico.zip': u'DC_TDC',
             u'DynamoBios-Network-Insilico.zip': u'Dynamo Bios',
             u'FUBerlin-Network-Insilico.zip': u'FUBerlin',
             u'Freya-Network-Insilico.zip': u'Freya',
             u'Frs5pB-Network-Insilico.zip': u'Frs5pB',
             u'GoatTower-Network-Insilico.zip': u'Goat Tower',
             u'Gupta2-Network-Insilico.zip': u'Gupta',
             u'HD_Systems7-Network-Insilico.zip': u'HD_Systems',
             u'Hairball-Network-Insilico.zip': u'Hairball',
             u'Hatric-Network-Insilico.zip': u'Hatric',
             u'Hogwarts2-Network-Insilico.zip': u'Hogwarts2',
             u'JCheng-Network-Insilico.zip': u'JCheng',
             u'KALEH-Network-Insilico.zip': u'KALEH',
             u'MA1-Network-Insilico.zip': u'DoNET',
             u'Morpheus-Network-Insilico.zip': u'Morpheus',
             u'NIPL-Network-Insilico.zip': u'NIPL',
             u'NMSUSongLab-Network-Insilico.zip': u'NMSUSongLab',
             u'Netzwerk-Network-Insilico.zip': u'Netzwerk',
             u'Pitt.transmed-Network-Insilico.zip': u'Pitt.transmed',
             u'Platinum-Network-Insilico.zip': u'Platinum',
             u'SBIT-Network-Insilico.zip': u'SBIT',
             u'SBI_Lab-Network-Insilico.zip': u'SBI_Lab',
             u'SENSE-Network-Insilico.zip': u'SENSE',
             u'SFTTRW-Network-Insilico.zip': u'SFTTRW',
             u'SUNQR-Network-Insilico.zip': u'SUNQR',
             u'ScreamingGoats-Network-Insilico.zip': u'ScreamingGoats',
             u'Shen-Network-Insilico.zip': u'Shen',
             u'Singularity-Network-Insilico.zip': u'Singularity',
             u'StuartLab-Network-Insilico.zip': u'StuartLab',
             u'TaylorSwift-Network-Insilico.zip': u'Taylor Swift',
             u'Tongji-Network-Insilico.zip': u'tongji team',
             u'UCSD_DING-Network-Insilico.zip': u'ding',
             u'WH-Network-Insilico.zip': u'WH',
             u'WelchsLab-Network-Insilico.zip': u'WelchsLab',
             u'Zhangroup-Network-Insilico.zip': u'Zhangroup',
             u'amss1012-Network-Insilico.zip': u'amss1012',
             u'bacbddepn-Network-Insilico.zip': u'bacbddepn',
             u'bdalab-Network-Insilico.zip': u'bdalab',
             u'biomecis2-Network-Insilico.zip': u'Biomecis',
             u'dftt-Network-Insilico.zip': u'dftt',
             u'guanlab18-Network-Insilico.zip': u'GuanLab',
             u'hibiscus-Network-Insilico.zip': u'hibiscus',
             u'insilico.zip': u'ICHING',
             u'limax-Network-Insilico.zip': u'limax',
             u'lpNet-Network-Insilico.zip': u'lpNet',
             u'sakev-Network-Insilico.zip': u'sakev',
             u'sannio2-Network-Insilico.zip': u'sannio2',
             u'sfntt-Network-Insilico.zip': u'sfntt',
             u'slugly-Network-Insilico.zip': u'slugly'}



def create_all_aggregation_figures():

    # SC1A, best, mean
    sc1a = SC1A_aggregation()
    try:
        pylab.figure(1)
        pylab.clf()

        sc1a.plot_aggr_best_score()
        pylab.savefig("sc1a_aggregation_best_mean.png")
        pylab.savefig("sc1a_aggregation_best_mean.svg")
        pylab.savefig("sc1a_aggregation_best_mean.pdf")
    except:
        print("SC1A best mean failed")

    # SC1A, best, mean
    try:
        pylab.figure(2)
        pylab.clf()
        sc1a.mode = "median"
        sc1a.plot_aggr_best_score()
        pylab.savefig("sc1a_aggregation_best_median.png")
        pylab.savefig("sc1a_aggregation_best_median.svg")
        pylab.savefig("sc1a_aggregation_best_median.pdf")
    except:
        print("SC1A best median failed")


     # SC1A, random, mean
    sc1a = SC1A_aggregation()
    try:
        pylab.figure(1)
        pylab.clf()

        sc1a.plot_aggr_random()
        pylab.savefig("sc1a_aggregation_random_mean.png")
        pylab.savefig("sc1a_aggregation_random_mean.svg")
        pylab.savefig("sc1a_aggregation_random_mean.pdf")
    except:
        print("SC1A random mean failed")


    # SC1A, random, mean
    try:
        pylab.figure(2)
        pylab.clf()
        sc1a.mode = "median"
        sc1a.plot_aggr_best_score()
        pylab.savefig("sc1a_aggregation_random_median.png")
        pylab.savefig("sc1a_aggregation_random_median.svg")
        pylab.savefig("sc1a_aggregation_random_median.pdf")
    except:
        print("SC1A random median failed")


def sc1a_check_edge_scores_range():
    """

    load all submissions from sc1a and check that values are >0.
    If not, print info


    values larger than one im amss1012, sannio2,ML_BHK_KG
    large values around 8 for ML_BHK_KG-N
    between 1.2 and 1.8 for sannio
    up to 30 for amss1012
    # need to comment scaling in socring module.
    """
    filenames = glob.glob(os.sep.join([d8c1path, "submissions", "sc1a", "*zip"]))
    for filename in filenames:
        print("#################")
        print filename
        s = scoring.HPNScoringNetwork(filename,verbose=False)
        p = pd.Panel(dict([(c+"_"+l, pd.DataFrame(s.edge_scores[c][l]))
            for c in s.edge_scores.keys() for l in s.edge_scores[c].keys()]))
        for item in p.items:
            if p[item].min().min()<0:
                print("-----Found negative in %s %s" % (filename, item))
            if p[item].max().max()>1:
                print("-------Found larger than one in %s %s" % (filename, item))
            #print("---------- max in %s %s" %(item, p[item].max().max()))


def sc1b_check_edge_scores_range():
    """Check that all SC1B edge score are in the range [0,1]

    sannio2 with values ~ 1.2
    # need to comment scaling in socring module.
    """
    filenames = glob.glob(os.sep.join([d8c1path , 'submissions', "sc1b", "*zip"]))
    for filename in filenames:
        print("#################")
        print filename
        s = scoring.HPNScoringNetworkInsilico(filename,verbose=False)
        p = pd.DataFrame(s.user_graph)

        if p.min().min()<-0.0001:
            print("-----Found NEGATIVE in %s " % (filename))
        if p.max().max()>1.001:
            print("-------Found larger than one in %s " % (filename))
        #print("---------- max in %s" %(p.max().max()))



# -*- python -*-
#
#  This file is part of DreamTools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.org/dreamtools
#
##############################################################################
from dreamtools.core.challenge import Challenge
from dreamtools.core.rtools import RTools
import os


__all__ = ['D8C2_sc2']

class D8C2_sc2(Challenge, RTools):
    """D8C2 Tox challenge scoring (sub challenge 2)

    ::

        from dreamtools import D8C2_s2
        s = D8C2_sc2(filename)
        s.run()
        s.df


    see `github README <https://github.com/dreamtools/dreamtools/tree/master/dreamtools/dream8/D8C2>`_ for details


    """

    def __init__(self, filename, verboseR=True):
        Challenge.__init__(self, challenge_name='D8C2')
        RTools.__init__(self, verboseR=verboseR)

        self.filename = filename
        self._path2data = os.path.split(os.path.abspath(__file__))[0]

    def run(self):
        """Compute the score and populates :attr:`df` attribute with results"""

        script = """
            load("%(path)sdata/data_sch2.RData")
            source("%(path)sfunctionsLeaderboard_sch2.R")
            source("%(path)sprob_Cidx.R")

            load("%(path)stestSubmissions_sch2.RData")
            yourSubmission = read.table("%(filename)s")
            #scoreByCompound<-computeByCompound(submission=randomSubmission2,
            #    observed=observed, metric="pearson", keepCompounds=toxCompoundsID)
            # example2, compute prob C-index for the best Performer (note: prob C-index
            # computation takes some time)
            #scoreByCompound<-computeByCompound(submission=bestPerformer, observed=observed,
            #    metric="probCindex", keepCompounds=toxCompoundsID, pooled_std=pooled_std)
            submissions<-list(bestPerformer=bestPerformer, randomSubmission1=randomSubmission1, 
                yourSubmission=yourSubmission)
            res <- computeOverallScores(submissions)
            """

        print("Running the scoring function. This may take a couple of minutes.")
        params = {'filename':self.filename, 'path':self._path2data + os.sep}
        self.session.run(script % params)
        self.df = self.session.res.copy()
        self.df.index = ['bestPerformer','randomSubmission','yourSubmission']
        self.df.columns =  [c.strip() for c in self.df.columns]
        return self.df





# -*- python -*-
#
#  This file is part of DreamTools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.org/dreamtools
#
##############################################################################
"""Provides tools related to Receiver Operating Characteristic (ROC).


Those code were direct translation of Perl or matlab codes but we stronly recommend
to use scikit-learn in the future."""
import numpy as np


__all__ = ['ROC', 'ROCDiscovery']


class ROCBase(object):
    """An ABC class"""

    def __init__(self):
        pass

    def get_statistics(self):
        raise NotImplementedError

    def compute_auc(self, roc=None):
        """Compute AUC given a ROC data set (recomputed otherwise)

        :param str roc: The roc data structure must be a dictionary with "tpr"
            key. Could be a variable returned by :meth:`get_statistics`. If not
            provided, compute the ROC.
        :return: the AUC

        """
        if roc == None:
            roc = self.get_statistics()
        import scipy.integrate
        value = scipy.integrate.trapz(roc['tpr'], roc['fpr'])
        return value
        #return sum(roc['tpr']) / len(roc['tpr'])

    def compute_aupr(self, roc=None):
        """Compute AUPR given a ROC data set (recomputed otherwise)

        :param str roc: The roc data structure must be a dictionary with "tpr"
            key. Could be a variable returned by :meth:`get_statistics`. If not
            provided, compute the ROC.
        :return: the AUPR
        """
        if roc == None:
            roc = self.get_statistics()
        import scipy.integrate
        value = scipy.integrate.trapz(roc['precision'], x=roc['recall'])
        return value


class ROC(ROCBase):
    """A class to compute ROC, AUC and AUPRs for a binary problem


        >>> r = ROC() # could provide scores and labels as arguments
        >>> r.scores = [0.9,0.8,0.7,.6,.6,.3]
        >>> r.classes = [1,0,1,0,1,1]
        >>> r.compute_auc()
        0.4375
    

    """
    def __init__(self, scores=None, classes=None):
        """.. rubric:: Constructor

        :param list scores: the scores
        :param list classes: binary class made of 1 or 0 numerical values. Also
            called labels in the literature.
        """
        super(ROC, self).__init__()
        self._scores = None
        self._classes = None

        self.scores = scores
        self.classes = classes

    def _set_scores(self, values):
        self._scores = values
    def _get_scores(self):
        return self._scores
    scores = property(_get_scores, _set_scores, doc="Read/Write the scores")

    def _set_classes(self, values):
        self._classes = values
    def _get_classes(self):
        return self._classes
    classes = property(_get_classes, _set_classes, doc="Read/Write the classes")

    def _compute_fpr_tpr(self, FP, TP, N, P):
        roc = {"fpr":[], "tpr":[], "FP":[], "TP":[]}
        roc['fpr'] = FP/float(N)
        roc['tpr'] = TP/float(P)
        roc['TP'] = TP
        roc['FP'] = FP
        return roc

    def get_roc(self):
        """See :meth:`get_statistics`"""
        return self.get_statistics()

    def get_statistics(self):
        """Compute the ROC curve X/Y vectors and some other metrics

        :return: a dictionary with different metrics such as FPR (false positive
            rate), PTR (true positive rate).

        """
        scores = self.scores
        classes = list(self.classes)

        sorted_indices = np.argsort(1-np.array(scores))
        sorted_scores = [scores[i] for i in sorted_indices]
        sorted_classes = [classes[i] for i in sorted_indices]
        P = float(classes.count(1))
        N = float(classes.count(0))
        FP = 0
        TP = 0
        #
        roc = {"fpr":[], "tpr":[], "precision":[], "FP":[], "TP":[],
            "recall":[], "accuracy":[], "Fmeasure":[], 'threshold':[]}
        fprev = 1.1 # scores are less than 1 

        # an efficient algorithm to compute ROC based on T.Fawcett Pattern
        # Recognition Letters 27, 2006
        i = 0
        while i<len(scores):
            cls = sorted_classes[i]
            score = sorted_scores[i]

            if score != fprev and cls!=None:
                thisroc = self._compute_fpr_tpr(FP, TP, N, P)
                for key in ['fpr', 'tpr', "FP", "TP"]:
                    roc[key].append(thisroc[key])
                roc['threshold'].append(fprev)
                fprev = score

            if cls == 1:
                TP += 1
            elif cls == 0:
                FP += 1
            elif cls == None or np.isnan(cls):
                pass
            else:
                raise ValueError("found a class value different from 0,1 or None")
            i += 1
        sroc = self._compute_fpr_tpr(FP, TP, N, P)
        for key in ['fpr', 'tpr', "FP", "TP"]:
            roc[key].append(thisroc[key])

        # force the last value of tpr/fpr to be one
        # TODO this is a hack to bebahve correctly when the final fpr/tpr are nt
        # (1,1) as it should be.
        roc['fpr'][-1] = 1
        roc['tpr'][-1] = 1

        roc['threshold'].append(sorted_scores[-1])

        roc = self._compute_other_metrics(roc)
        return roc

    def _compute_other_metrics(self, roc):
        #Be aware that there is alway a first value TP=0,FP=0
        # this should be handled with care the recall/precision computatio
        # This first value is used to compute fpr and tpr but should not be used 
        # here
        FP = np.array([float(x) for x in roc['FP']])
        TP = np.array([float(x) for x in roc['TP']])
        N = max(FP) + max(TP)

        # The first value of precision is undefined since FP+TP=0
        # It does not matter since it will be overwritten as explained here
        # below

        # TODO: to prevent warning, set FP[0] to 100.
        FP[0] = 100
        precision = TP / (FP + TP)
        recall = TP/float(max(TP))
        # The second value may also be be an issue. There are 2 cases.
        # 1. TP>0
        # 2. TP == 0 
        # In the second case,  TP=0 and FP==0 should not happen. FP > 0 instead.
        # In which case, recall = 0 and precision=0. No ambiguity
        # In the first case, if TP>0 then precision >0 and recall > 0. 
        # Therefore we end up without a defined value at recall==0
        # So we should set a recall=0 for which precision will be the same as
        # the one where recall >0
        # somehow, precision and recall 
        if TP[1]>0:
            precision[0] = precision[1] 
        else:
            precision[0] = 0

        # TODO
        #roc['accuracy'] = (TP+FP)/(P+N)
        #if roc['recall'] >0 and roc['precision'] != np.nan and roc['precision'] > 0 :
        #    roc['Fmeasure'] = 2./(1./roc['precision']+1./roc['recall'])
        #else:
        #    roc['Fmeasure'] = np.nan
        roc['precision'] = [float(x) for x in precision]
        roc['recall'] = [float(x) for x in recall]
        return roc

    def plot_roc(self, roc=None):
        """Plot ROC curves

        .. plot::
            :include-source:
            :width: 80%

            from dreamtools.core.rocs import ROC
            r = ROC()
            r.scores = [.9,.5,.6,.7,.1,.2,.6,.4,.7,.9, .2]
            r.classes = [1,0,1,0,0,1,1,0,0,1,1]
            r.plot_roc()

        """
        if roc == None:
            roc = self.get_statistics()
        from pylab import plot, xlim, ylim ,grid, title, xlabel, ylabel
        x = roc['fpr']
        plot(x, roc['tpr'], '-o')
        plot([0,1], [0,1],'r')
        ylim([0, 1])
        xlim([0, 1])
        grid(True)
        title("ROC curve (AUC=%s)" % self.compute_auc(roc))
        xlabel("FPR")
        ylabel("TPR")


class ROCDiscovery(ROCBase):
    """A variant of ROC statistics

    Used in D5C2 challenge.

    .. note:: Does not work if any NA are found.
    """
    def __init__(self, discovery):
        """.. rubric:: constructor


        :param discovery: a list of 0/1 where 1 means positives
        """
        super(ROCDiscovery, self).__init__()

        try:
            self.discovery = discovery.values # a pandas time series ?
        except:
            self.discovery = np.array(discovery)

        # sanity checks
        assert set(np.unique(self.discovery)) == set([0,1]), 'ERROR: discovery is only allowed to have (0,1) entries.'

    def get_statistics(self):
        """Return dictionary with FPR, TPR and other metrics"""
        discovery = self.discovery # an alias
        T = len(discovery)  # Total
        P = np.sum(discovery) # positives
        N = T - P # negatives

        # true positives (false positives) at depth k
        TPk = np.cumsum(discovery)
        FPk = np.cumsum(1-discovery)

        # metrics
        TPR = TPk / float(P)
        FPR = FPk / float(N)
        REC = TPR  # recall
        PREC = TPk / np.linspace(1, T+1,T) # precision

        roc = {"fpr": FPR, "tpr":TPR, "precision":PREC, "FP":[], "TP":[],
            "recall": REC, "accuracy":[], "Fmeasure":[], 'threshold':[]}
        return roc

    def compute_aupr(self, roc=None):
        """Returns AUPR normalised by (1-1/P) P being number of positives"""
        AUPR = super(ROCDiscovery, self).compute_aupr(roc=roc)
        P = np.sum(self.discovery)
        return AUPR / (1-1./P)








# -*- python -*-
#
#  This file is part of DreamTools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.org/dreamtools
#
##############################################################################
"""Scoring Functions for the Dream7 Challenge 1 (Parameter estimation and network topology prediction)


:References:
    * http://dreamchallenges.org/project-list/dream7-2012/
    * https://www.synapse.org/#!Synapse:syn2821735/wiki/

:Publications: http://www.biomedcentral.com/1752-0509/8/13/abstract

"""
from dreamtools.core.challenge import Challenge

import numpy as np
import glob
import pandas as pd
import os


__all__ = ['D7C1']



class D7C1(Challenge):
    """DREAM 7 - Network Topology and Parameter Inference Challenge

    Here is a quick example on calling the scoring methods::

        from dreamtools import D7C1
        s = D7C1()
        s.score_model1_timecourse(filename)
        s.score_model1_parameters(filename)
        s.score_topology(filename)


    This class provides 3 main scoring functions:

    #. :meth:`score_topology`
    #. :meth:`score_model1_timecourse`
    #. :meth:`score_model1_parameters`

    Each takes as an input a valid submission as described in the official
    `synapse page <https://www.synapse.org/#!Synapse:syn2821735/wiki/>`_.

    Templates are also provided within the source code on `github dreamtools <https://github.com/dreamtools>`_
    in the directory dreamtools/dream7/D7C1/templates.

    D7C1 scoring function are also included in the standalone code **dreamtools-scoring**.

    For the details of the scoring functions, please refer to the paper (see module documentation)
    Some details are provided in the methods themselves as well.

    There are other methods (starting with leaderboard) that should not be used. Those are
    draft version used to compute pvalues and report scores as in the final leaderboard.

    .. note:: the scoring functions were implemented following Pablo Meyer's matlab **codescore_dream7_c1s1.m**
    """

    def __init__(self, path='submissions'):
        """.. rubric:: constructor

        :param path: path to a directory containing submissions
        :return:
        """
        Challenge.__init__(self, challenge_name='D7C1')

        self.path = path

        teams = glob.glob(path + os.sep +'*')
        self.teams = [this for this in teams if os.path.isdir(this) is True]

        self.N = len(self.teams)
        self.distances = np.zeros((self.N, 4))
        self.pvalues = np.zeros((self.N, 4))

        self.scores = {}
        self._load_gold_standard()

        #self.load_submissions()
        #self.compute_score_distance_model1()
        #self.compute_score_parameter_prediction_model1()
        #self.compute_score_topology()
        
        # data structure to store null distances
        self.rdistance_pred1 = [] 
        self.rdistance_param1 = []

    def load_submissions(self):
        """Load a bunch of submissions to be found in the submissions directory

        The directory name is defined in :attr:`path`


        :return: nothing. Populates :attr:`data` attribute and :attr:`team_names`.
        """

        tag_param1 = 'dream7_netparinf_parameters_model_1'
        tag_param2 = 'dream7_netparinf_parameters_model_2'
        tag_pred1 = 'dream7_netparinf_timecourse_model_1'
        tag_topo2 = 'dream7_netparinf_networktopo_model_2'

        self.data = {}
        self.data['param1'] = {}
        self.data['param2'] = {}
        self.data['pred1'] = {}
        self.data['topo2'] = {}
        self.team_names = []

        for team in self.teams:
            team_name = team.split(os.sep)[1]
            self.team_names.append(team_name)
            
            filename = team + os.sep + tag_param1 + '_' + team_name + '.txt'
            self.data['param1'][team_name] = self._read_df(filename, 'param')
        
            filename = team + os.sep + tag_param2 + '_' + team_name + '.txt'
            self.data['param2'][team_name] = self._read_df(filename, 'param')

            filename = team + os.sep + tag_pred1 + '_' + team_name + '.txt'
            self.data['pred1'][team_name] = self._read_df(filename, 'pred')

            filename = team + os.sep + tag_topo2 + '_' + team_name + '.txt'
            self.data['topo2'][team_name] = self._read_df(filename, 'topo')

    def _read_df(self, filename, mode, sep='\s+'):

        def error_message(msg):
            txt = 'Error while parsing %s\n' % filename
            txt += msg
            return txt
        if mode == 'param':
            df = pd.read_csv(filename, sep=sep, index_col=0,
                header=None, names=['values'])
        elif mode == 'pred':
            df = pd.read_csv(filename, sep=sep, index_col=0)
            assert all(df.columns == ['p3','p5','p8']), \
                error_message('Header columns  must be p3 p5 p8. Found %s' % df.columns)
        elif mode == 'topo':
            df = pd.read_csv(filename, sep=sep, index_col=None, header=None,
                    names=['regulator','sign1','g1','sign2','g2'])

        return df

    ############################################################################   standalone scoring functions:
    def score_model1_parameters(self, filename):
        r"""Return distance between submission and gold standard for parameters challenge (model1)

        :param filename: must be valid templates
        :return: score (distance)


        ::

            >>> from dreamtools import D7C1
            >>> s = D7C1()
            >>> s.score_model1_parameters('templates/model1_parameters_alphabeta.txt')
            0.022867555017785129


        The score is computed using the square of the ratio of the user prediction and the gold standard.
        Taking the mean of the log10 :

        .. math::

            S = \overline{\log10 \left( \left( \frac{X}{X_{\rm{gold\;standard}}} \right)^2\right)}


        """
        data = self._read_df(filename, mode='param')
        distance = self._compute_score_model1_parameters(data)
        return distance

    def score_model1_timecourse(self, filename):
        r"""Returns distance between prediction and gold standard (model1)


        :param filename: must be valid templates
        :return: score (distance)

        ::

            >>> from dreamtools import D7C1
            >>> s = D7C1()
            s.score_model1_timecourse('templates/model1_timecourse_alphabeta.txt')
            0.0024383612676804048


        There are 3 time courses to be predicted.
        The score for each time course is

        .. math::

            S_i = \frac{(X_i - \hat{X_i}) ^ 2}{0.01 + 0.04 * X_i^2}

        where :math:`X` is the gold standard and :math:`\hat{X}` the prediction.
        and final score is just the average across the 3 time courses.

        """
        data = self._read_df(filename, mode='pred')
        distance = self._compute_score_timecourse_model1(data, 10,39)
        return distance

    def score_topology(self, filename):
        """Standalone version of the network topology scoring

        :param str filename:

        ::

            >>> from dreamtools import D7C1
            >>> s = D7C1()
            >>> s.score_topology('templates/network_topology_alphabeta.txt')
            12


        :scoring details:

            The challenge requests predictions for 3 missing links, knowing that
            a gene can regulate up to two genes when they are in the same operon,
            6 gene interactions have to be indicated by the participants (3 links*2 genes)
            and whether these interactions are activating (+) or repressing (-).

            For each of the predicted links i=1,2,3, we define a score:

            :math:`S_i^{link} = L_i + N_i`

            where :math:`L_j` is 6 if the nature of
            the regulation iscorrect (that is, the source gene, the sign of the connection, and
            the destination gene are all correct) and :math:`L_i = 12` if the link
            regulates an operon composed of two genes and both connections are
            correct. If :math:`L_i >0` then :math:`N_i=0`.

            In case a link is NOT correctly predicted (:math:`L_i=0`) :math:`N_i`
            is defined as follows. It is increased by 1 for each correctly
            regulated gene, 2 if the regulated gene and the nature of the
            regulation (i.e +/-) are correct and 1 if the regulator gene is
            correct


            The gold standard contains 3 lines similar to ::

                5 + 7 + 11

            It means gene 5 positively regulates gene 7 and gene 11.  If a prediction is ::

                5 + 7 + 2

            Then L =6. If the prediction is ::

                2 + 7 + 2

            L = 0 so N may be updated. Here the regulon (2) is not correct, However, one gene (7) is correctly predicted
            with the good sign so N = 2.


        """
        data = self._read_df(filename, mode='topo')
        distance = self._compute_score_topology(data)
        return distance

    ############################################################################# Load gold standard files
    def _get_gs(self, filename):
        self._path2data = os.path.split(os.path.abspath(__file__))[0]
        filename = os.sep.join([self._path2data, "goldstandard", filename])
        return filename

    def _load_gold_standard(self):
        self.gs = {}
        self.gs['param1'] = self._read_df(self._get_gs("model1_parameters_answer.txt"), mode='param')
        self.gs['param2'] = self._read_df(self._get_gs("model2_parameters_answer.txt"), mode='param')
        self.gs['pred1'] = self._read_df(self._get_gs("model1_prediction_answer.txt"), mode='pred')
        self.gs['topo2'] = self._read_df(self._get_gs("model2_topology_answer.txt"), mode='topo')


    ############################################################################# compute all submissions from challenge
    def leaderboard_compute_score_topology(self):
        """Computes all scores (topology) for loaded submissions

        For the metric, see :meth:`score_topology`.

        :return: fills :attr:`scores`.


        .. seealso:: :meth:`load_submissions`
        """
        scores = {}
        for team in self.team_names:
            data = self.data['topo2'][team]
            score = self._compute_score_topology(data, team=team)
            scores[team] = score
        df = pd.TimeSeries(scores)
        df = pd.DataFrame({'scores':df, 'rank':df.rank()})
        self.scores['topo2'] = df.sort(columns='rank')

    def leaderboard_compute_score_parameters_model1(self):
        """Computes all scores (parameters model1)

        :return: Nothing but fills :attr:`scores`.

        For the metric, see :meth:`score_model1_parameters`.


        .. seealso:: :meth:`load_submissions`

        """
        # parameter model1
        scores = {}
        for team in self.team_names:
            data = self.data['param1'][team]
            score = self._compute_score_model1_parameters(data)
            scores[team] = score
        df = pd.TimeSeries(scores)
        df = pd.DataFrame({'scores':df, 'rank':df.rank()})
        self.scores['param1'] = df.sort(columns='rank')

    def leaderboard_compute_score_timecourse_model1(self, startindex=10, endindex=39):
        """Computes all scores (timecourse model1)

        :return: Nothing but fills :attr:`scores`

        For the metric, see :meth:`score_model1_parameters`.

        Note that *endindex* is set to 39 so it does not take into account last value at time=20
        This is to be in agreement with the implemenation used in the final leaderboard

        https://www.synapse.org/#!Synapse:syn2821735/wiki/71062

        If you want to take into account final point, set endindex to 40

        """
        scores = {}
        for team in self.team_names:
            data = self.data['pred1'][team]
            scores[team] = self._compute_score_timecourse_model1(data, startindex, endindex)
        self.scores['pred1'] = pd.TimeSeries(scores)
        self.scores['pred1'].sort()
        self.scores['pred1'] = self.scores['pred1'].to_frame()
        self.scores['pred1'].columns = ['scores']

    def _compute_score_model1_parameters(self, data):
        diff = data / self.gs['param1']
        diff = (np.log10(diff)**2).mean()
        score = diff.values[0] # should be a single float
        return score

    #@do_profile()
    def _compute_score_timecourse_model1(self, data, startindex, endindex):
        d1 = (self.gs['pred1'] - data) ** 2  
        d1 /= (0.01 + 0.04 * self.gs['pred1'].values**2)
        # let us ignore the first 10 points

        # faster to use numpy array and indices
        data = d1.values[startindex:endindex+1,:]
        N = endindex - startindex + 1.
        distance = np.sum(data) / (3*N)  # normalisation
        return distance


    #################################################################### NULL distribution (draft do not use)
    def get_null_parameters_model1(self, N=10000, Nbest=9):
        """Null distribution for the model1 parameter

        :param N: number of distribution
        :param int Nbest: In the official challenge, 12 submissions wre provided. Here, we use only the 9 best submissions
            like in the paper.
        :return: a dataframe with null distributions


        .. note:: submissions are loaded and scored before creating the null
            distributions.

        """
        self.load_submissions()
        self.leaderboard_compute_score_parameters_model1()
        # select only best 9 as in the paper
        Nbest = 9
        best_teams = list(self.scores['param1'].ix[0:Nbest].index)
        
        # create a dataframe to hold all teams and 
        p = pd.DataFrame(dict([(key, self.data['param1'][key].T.values[0]) 
            for key in best_teams]))

        nulls = []
        for k in xrange(0,45):
            null = p.ix[k][np.random.randint(0, Nbest, N)]
            nulls.append(null)

        indexnames = list(self.gs['param1'].index)
        df = pd.DataFrame(dict([(name, nulls[i].values) 
            for i, name in enumerate(indexnames)]))
        df = df[self.gs['param1'].index]
        return df

    def _compute_rdist_param1(self, N=10000):
        df = self.get_null_parameters_model1(N=N)

        distances =[]
        from easydev import progress_bar
        pb = progress_bar(N)
        for i in xrange(0, N):
            df1 = df.ix[i].to_frame(name='values')
            distance = self._compute_score_model1_parameters(df1)
            distances.append(distance)
            pb.animate(i, 0)
        return distances

    def _compute_pvalues_param1(self, N=10000):
        rdist = self._compute_rdist_param1(N=N)
        self.rdistance_param1.extend(rdist)
        #pvalues = [len([x for x in self.rdistance_param1 if x <=score] )/float(N) 
        #        for score in self.scores['param1'].scores]
        #self.scores['param1']['pvalues'] = pvalues

    def get_null_timecourse_model1(self, N=10000, Nbest=9):
        """Null distributions for the model1 timecourse challenge

        :param N: number of distributions
        :param int Nbest: In the official challenge, 12 submissions wre provided. Here, we use only the 9 best submissions
            like in the paper.
        :return: a dataframe with the null distribution
        """

        best_teams = list(self.scores['pred1'].ix[0:Nbest].index)
        print(best_teams)

        # data mangling to extract random values easily 
        p3 = pd.DataFrame(dict([(key, self.data['pred1'][key]['p3']) for key in best_teams]))
        p5 = pd.DataFrame(dict([(key, self.data['pred1'][key]['p5']) for key in best_teams]))
        p8 = pd.DataFrame(dict([(key, self.data['pred1'][key]['p8']) for key in best_teams]))

        df = self.gs['pred1'].copy()

        data = np.zeros((41,3, N))
        for ik,k in enumerate(list(self.gs['pred1'].index)):
            data[ik,0] = p3.ix[k][np.random.randint(0,Nbest,N)]
            data[ik,1] = p5.ix[k][np.random.randint(0,Nbest,N)]
            data[ik,2] = p8.ix[k][np.random.randint(0,Nbest,N)]
        return data
           
    #@do_profile()
    def _compute_rdist_pred1(self, N=10000):
        data = self.get_null_timecourse_model1() # numpy matrices

        distances = []
        from easydev import progress_bar
        pb = progress_bar(N)
        for i in xrange(0,N):
            df = data[:,:,i]
            # FIXME those values 10,39 should not be hardcoded
            distance = self._compute_score_timecourse_model1(df, 10,39)
            distances.append(distance)
            pb.animate(i, 0)
        return distances

    def _compute_pvalues_pred1(self ,N=10000):
        rdist = self._compute_rdist_pred1(N=N)
        self.rdistance_pred1.extend(rdist[:])
        #pvalues = [len([x for x in self.rdistance_pred1 if x <=score] )/float(N) 
        #        for score in self.scores['pred1'].scores]
        #self.scores['pred1']['pvalues'] = pvalues

    def _leaderboard_compute_overall_score(self, N=100):
        """Based on NULL distribution, compute overall score of model1

        Not finalised.

        """
        self._compute_pvalues_pred1(N=N)
        self._compute_pvalues_param1(N=N)
        import fitter
        fit_param1 = fitter.Fitter(self.rdistance_param1)
        fit_param1.distributions = ['beta']
        fit_param1.fit()
        fit_pred1 = fitter.Fitter(self.rdistance_pred1)
        fit_pred1.distributions = ['beta']
        fit_pred1.fit()

        import scipy.stats
        self.pvalues_param1 = scipy.stats.beta.cdf(self.scores['param1'].scores, 
                *fit_param1.fitted_param['beta'])
        self.pvalues_pred1 = scipy.stats.beta.cdf(self.scores['pred1'].scores, 
                *fit_pred1.fitted_param['beta'])

        self.scores['pred1']['pvalues'] = self.pvalues_pred1
        self.scores['param1']['pvalues'] = self.pvalues_param1


    def leaderboard(self):
        """Computes all scores for all submissions and returns dataframe


        :return: dataframe with scores for each submissions for the
            model1 (parameter and timecourse) and model2 (topology)
        """
        self.load_submissions()
        self.leaderboard_compute_score_parameters_model1()
        self.leaderboard_compute_score_topology()
        self.leaderboard_compute_score_timecourse_model1(10,39)

        df = pd.merge(self.scores['param1'], self.scores['pred1'],
                left_index=True, right_index=True, 
                suffixes=['_parameter', '_timecourse'])


        df = pd.merge(df, self.scores['topo2'],
                left_index=True, right_index=True
                )
        df.columns = [c if c!='scores' else 'scores_topology' for c in df.columns]

        return df


    def _compute_score_topology(self, data, team=''):
        """see :meth:`score_topology` for details """
        data = data.copy() # make sure we do not change the data so use copy()
        gs = self.gs['topo2'].copy()

        Li = np.array([0,0,0])
        Ni = np.array([0,0,0])

        # loop over solution
        cols1 = ['regulator', 'sign1', 'g1']
        cols2 = ['regulator', 'sign2', 'g2']
        # build list of existing genes and their signs from the GS
        # order does not matter, this is for th counting of Ni
        genes =  pd.concat([gs.g1 , gs.g2]).values
        signs =  pd.concat([gs.sign1 , gs.sign2]).values

        # make sure there are unique
        regulators_data = set(data.regulator)


        for i in range(0,3):
            for j in range(0,3):
                # if all 5 values are correct, L = 12 and stops there
                # Note that is the regulated gene is zero, it means 
                # it does not exists so it is ignored.
                if all(gs.ix[i][cols1] == data.ix[j][cols1]) is True:
                    # g1 should be different from 0
                    # regulator is tested as well although the GS are no such case
                    if gs.ix[i]['g1'] != 0 and gs.ix[i]['regulator'] != 0 :
                        Li[i] += 6
                        data.iloc[j,0] = 0 # index 0 means first columns that is the regulator
                        data.iloc[j,2] = 0 # index 4 means gene1
                if all(gs.ix[i][cols2] == data.ix[j][cols2]) is True:
                    # g1 should be different from 0
                    # regulator is tested as well although the GS are no such case
                    if gs.ix[i]['g2'] != 0 and gs.ix[i]['regulator'] != 0 :
                        Li[i] += 6
                        data.iloc[j,0] = 0 # index 0 means first columns that is the regulator
                        data.iloc[j,4] = 0 # index 4 means gene2

        # any regulator correctly predicted ?

        for reg in set(data.regulator):
            if reg !=0 and reg in gs.regulator.values:
                Ni[i] += 1

        genes_data =  pd.concat([data.g1 , data.g2]).values
        signs_data =  pd.concat([data.sign1 , data.sign2]).values
        import collections

        dgenes = collections.defaultdict(list)
        for gene, sign in zip(genes,signs):
            dgenes[gene].append(sign)
        dgenes_data = collections.defaultdict(list)
        for gene, sign in zip(genes_data,signs_data):
            dgenes_data[gene].append(sign)

        # here this is ambigous: a gene may have 2 opposite signs... To be in agreement with original code
        # from pablo that was used in the dream challenge, we keep the latest values found in the submissions.
        # this is arbitrary though.
        #for key in dgenes_data.keys():
        #  if dgenes_data.
        del dgenes[0]
        del dgenes_data[0]

        # any gene and sign found in GS?
        for gene, signs in dgenes_data.iteritems():
            if gene in dgenes.keys():
                Ni[i] += 1
                if signs[-1] in dgenes[gene]:
                    Ni[i] += 1


        Si = Li + Ni
        return sum(Si)




# -*- python -*-
#
#  This file is part of dreamtools software
#
#  Copyright (c) 2011-2014 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.com/dreamtools
#
##############################################################################
from distutils.core import setup
from distutils.extension import Extension
from Cython.Distutils import build_ext

include_dirs = []
ext_modules = [Extension("cython_scoring", ["cython_scoring.pyx"], include_dirs)
              ]

setup(
  name = 'optimisation cython',
  cmdclass = {'build_ext': build_ext},
  ext_modules = ext_modules
)





# -*- python -*-
#
#  This file is part of DreamTools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.org/dreamtools
#
##############################################################################
"""Common utility to all challenges"""

import os
import re
from dreamtools import configuration as cfg


class Challenge(object):
    """Handler for the creation of config directory for each challenge"""

    def __init__(self, challenge_name):
        """.. rubric:: constructor

        :param str challenge_name:

        """
        #: nickname of the challenge. Must be DXCY form with X, Y being 2 numbers
        self.nickname = challenge_name
        self._check_challenge_name()
        #: directory where is stored the configuration file and possibly data files.
        self.mainpath = cfg.user_config_dir

    def _get_directory(self):
        """Gets directory where data will be stored."""
        name_dir = self._get_challenge_directory()
        directory = os.sep.join([self.mainpath, name_dir, self.nickname])
        return directory
    directory = property(_get_directory)

    def _check_challenge_name(self):
         results = re.search("^D\d+C\d+$", self.nickname)
         if results is None:
            msg = "Challenge name provided (%s) is incorrect expects DXCY with X and Y as numbers"
            raise ValueError(msg % name)

    def _get_challenge_directory(self):
        # name is e.g., D1C2 we want to extract the 1 (with more than 1 digit)
        name = self.nickname[1:] # get rid of first letter D
        version = name.split("C")[0]
        return "dream" + str(version)

    def mkdir(self):
        """Creates directory if does not exist already"""
        directory = self.get_directory()
        if os.path.exists(directory) is False:
            os.mkdir(directory)
        


